% function:  compare_coefs_vlahovska_here
% This function compares the coefficients of the evolution eqn for the second and
% fourth order deformation tensor between our paper and Vlahovska et al, JFM 2009 and Barthes-Biesel and
% Acrivos, JFM 1973.

% INPUT:  lambda = viscosity ratio
% OUTPUT:  error between the coefficients computed by us and BB's paper.

%            If the droplet radius is r = 1 + Ca * (r*D*r) + Ca^2 * (rr** D4 ** rr - 2/15 D**D)

%          The evoluation eqn for D is (d/dt is co-rotational derivative):
%          dD/dt = a_E*E + a_D*D + Ca * ( Sd2[E*D]*a_DE + Sd2[a_DD]*a_DD )
%                  + Ca^2 * a_E_DD * E * (D**D) + Ca^2 * a_D_DE * D *(D**E)
%                  + Ca^2 * a_D_DD * D * (D**D) + Ca^2 * a_EDD * Sd2[E*D*D]
%                  + Ca^2 * a_D4E * D4 ** E + Ca^2 * a_D4D * D4 ** D

%          The evoluation eqn for D4 is:
%          dD4/dt = b_D*D4 + b_DD * Sd4[DD] + b_DE * Sd4[ED]

%          We output the error between our calculated coefs and
%          Barthes-Biesel's

function [err_a_E, err_a_D, err_a_ED, err_a_DD, err_a_E_DD, err_a_D_DE, err_a_D_DD, ...
          err_a_EDD, err_a_ED4, err_a_D_D4, err_b_D4, err_b_ED, err_b_DD] = comp_coefs_vlahovska_here(lambda)

% Get coefficients from Vlahovska
[a_E_ac, a_D_ac, a_ED_ac, a_DD_ac, a_E_DD_ac, a_D_DE_ac, a_D_DD_ac, a_EDD_ac, a_ED4_ac, a_D_D4_ac, b_D4_ac, b_ED_ac, b_DD_ac] = get_coefs_vlahovska(lambda);

% Get coefficients aE, aD, aDE, aDD, bD4, b_ED, b_DD
[~, ~, ~, ~, ~, ~, ~, ~, ~, ...
       a_E, a_D, a_DD, a_ED, ...
       b_D4, b_DD, b_ED] = compute_coefs_ord_Ca(lambda, 0, 0);


% Get coeffidients for O(Ca^2)
%[~, ~, ~, ~, ~, ~, coefs_EDD, coefs_E_DD, coefs_D_DE, coefs_ED4, coefs_D_DD, coefs_D_D4] = calc_coefs_use_entire_vel(lambda, 0, 0);
[coefs_EDD, coefs_E_DD, coefs_D_DE, coefs_ED4, coefs_D_DD, coefs_D_D4] = calc_coefs_ord_Ca_sq(lambda, 0, 0);
a_EDD = coefs_EDD(5);
a_E_DD = coefs_E_DD(5);
a_D_DE = coefs_D_DE(5);
a_ED4 = coefs_ED4(5);
a_D_DD = coefs_D_DD(5);
a_D_D4 = coefs_D_D4(5);


% Compare coefficients
err_a_E = a_E - a_E_ac;
err_a_D = a_D - a_D_ac;
err_a_ED = a_ED - a_ED_ac;
err_a_DD = a_DD - a_DD_ac;
err_a_E_DD = a_E_DD - a_E_DD_ac;
err_a_D_DE = a_D_DE - a_D_DE_ac;
err_a_D_DD = a_D_DD - a_D_DD_ac;
err_a_EDD = a_EDD - a_EDD_ac;
err_a_ED4 = a_ED4 - a_ED4_ac;
err_a_D_D4 = a_D_D4 - a_D_D4_ac;
err_b_D4 = b_D4 - b_D4_ac;
err_b_ED = b_ED - b_ED_ac;
err_b_DD = b_DD - b_DD_ac;

end
