% Get coefficients from Vlahovksa, JFM 2009 and Barthes-Biesel and Acrivos, JFM 1973
% INPUT:  x = lambda
% OUTPUT:  coefficients for evolution eqn for the second and fourth order
%          deformation tensor.  We are using the convention in our JFM
%          paper, where droplet radius is r = 1 + Ca * (r*D*r) + Ca^2 * (rr** D4 ** rr - 2/15 D**D)
%                  D_here = 3 * D_acrivos, D4_here = 105 * D_acrivos

%          The evoluation eqn for D is (d/dt is co-rotational derivative):
%          dD/dt = a_E*E + a_D*D + Ca * ( Sd2[E*D]*a_DE + Sd2[a_DD]*a_DD )
%                  + Ca^2 * a_E_DD * E * (D**D) + Ca^2 * a_D_DE * D *(D**E)
%                  + Ca^2 * a_D_DD * D * (D**D) + Ca^2 * a_EDD * Sd2[E*D*D]
%                  + Ca^2 * a_D4E * D4 ** E + Ca^2 * a_D4D * D4 ** D

%          The evoluation eqn for D4 is:
%          dD4/dt = b_D*D4 + b_DD * Sd4[DD] + b_DE * Sd4[ED]

function [a_E, a_D, a_DE, a_DD, a_E_DD, a_D_DE, a_D_DD, a_EDD, a_D4E, a_D4D, b_D, b_DE, b_DD] = get_coefs_vlahovska(x)

a_E = 3 *  5/(3*(2*x + 3));
a_D =  -40*(x+1)/((2*x+3)*(19*x+16));
a_DE =  10*(4*x - 9)/(7*(2*x + 3)^2);
a_DD =  1/3 * 288*( 137*x^3 + 624*x^2 + 741*x + 248)/(7*(2*x + 3)^2 * (19*x + 16)^2);

a_E_DD = 1/3 * -2 * (11172*x^4 + 18336*x^3 + 17440*x^2 + 3499*x - 7572)/( 49*(2*x+3)^3 * (19*x+16) );

a_D_DE = 1/3 * -2*(x-1) * (22344*x^3 + 52768*x^2 + 45532*x + 19356)/( 49*(2*x+3)^3 * (19*x + 16) );

a_D_DD = -1/9 * 48* P(x)/(49* (2*x+3)^3 * (19*x+16)^3 * (10*x + 11) * (17*x + 16) );

a_EDD = 1/3 * 48 * (x-1) * (2793*x^3 + 7961*x^2 + 8474*x + 3522)/(49 * (2*x+3)^3 * (19*x + 16) );

a_D4E = 3/105 * -400 * (43*x^2 + 79*x + 53)/( 3 * (2*x+3)^2 * (19*x + 16) );

% NOTE:  There's a typo in a_D4D in BB's original paper.  Use Vlahovksa's
% 2009 JFM paper as correction (note eqn 8.6 in Vlahovka's paper needs to be multiplied by
% 105 in order to be correct using her notation.  We verified this via personal communication with the authors)
a_D4D = 80*Q_updated(x)/( (2*x+3)^2 * (19*x+16)^2 * (10*x + 11) * (17*x + 16) );
%a_D4D_acrivos = 1/105 * 80*Q(x)/( (2*x+3)^2 * (19*x+16)^2 * (10*x + 11) * (17*x + 16) );

b_D = -360 * (x+1)/( (17*x + 16) * (10*x + 11) );

b_DE = 105 * 1/(7*(2*x+3));

% NOTE:  There's a typo in b_DD in BB's original paper.  Use Vlahovksa's
% 2009 JFM paper as correction
b_DD = 105/3 * 16/21 * (-14*x^3 + 207*x^2 + 413*x + 192)/( (2*x+3)*(19*x+16) * (17*x + 16) * (10*x + 11) );
%b_DD_acrivos = 105/3 * 16/21 * (-14*x^3 + 207*x^2 + 431*x + 192)/( (2*x+3)*(19*x+16) * (17*x + 16) * (10*x + 11) );%

end


function sol = P(x)
sol =  2127976*x^7 - 16341920*x^6 - 38494964 * x^5 + 122942551 * x^4 + 474068311 * x^3 + 591515680 * x^2 + 332123136 * x + 71700480; 
end

function sol = Q(x)
sol = 405260 * x^5 + 2366960*x^4 + 9142173 * x^3 + 8595967 * x^2 + 3334160 * x + 693760;
end

function sol = Q_updated(x)
chi = x + 1;
sol = 81052*chi^5 + 68132*chi^4 + 10203*chi^3 - 20850*chi^2 + 135*chi + 80;
sol = sol/21;
end
