% Calculate velocity field up to O(Ca) and evaulate the boundary conditions on deformed surface

% INPUT:  E = rate of strain tensor
%         D = deformation of surface (2nd order tensor)
%         D4 = deformation of surface (4th order tensor)
%         phi, theta = azimuthal and polar angles
%         lambda = viscosity ratio
%         Bq_mu = Boussinesq number for shear viscosity
%         Bq_k = Boussinesq number for dilatational viscosity


% OUTPUT:  P_u = P * (uout - uin) = jump in tangential  velocity field across deformed surface
%          n_u = n * (uout - uin) = jump in normal  velocity field across deformed surface 
%          kin_BC = kinematic BC on deformed surface
%          P_sigma_n = [P*sigma_out*n - P*sigma_in*n] - Bq_mu * P * fvis - Bq_k * P * fdil =
%                      jump in tangential traction and surface stresses across deformed surface
%          n_sigma_n = n*[sigma_out - sigma_in]*n - Bq_mu * n* fvis - Bq_k * n * fdil = jump in normal 
%                                                   traction across deformed surface

function [P_u, n_u, kin_BC, P_sigma_n, n_sigma_n] = eval_BCs_updated_v2(E, D, D4, theta, phi, Ca, lambda, Bq_mu, Bq_k)

% Calculate coefficients for O(Ca) flow
[c_E, c_D, c_Sd2_ED, c_Sd2_DD, ...
          c_D4, c_Sd4_ED, c_Sd4_DD, ...
          c_Sd3_ED, c_Sd1_ED] = compute_coefs_ord_Ca(lambda, Bq_k, Bq_mu);
      
      
% Get geometric features of surface
[n, x, ~, Dn, ~, ~, dH, grad_g] = update_geom(D, D4, Ca, theta, phi);

% Get irreducible tensors
Sd2_ED = Sd2(E*D);
Sd4_ED = Sd4(E, D);
Sd2_DD = Sd2(D*D);
Sd4_DD = Sd4(D, D);
[Sd1_ED, Sd3_ED] = get_pseudo_tensors(E,D);


% Get velocity driven by E
[u_out_E, du_out_E, u_in_E, du_in_E, ddu_in_E, p_out_E, p_in_E] = eval_dist_velocity_fields_second_ord_analytical(E, x, lambda, c_E);

% Get velocity driven by Sd1(ED)
[u_out_Sd1_ED, du_out_Sd1_ED, u_in_Sd1_ED, du_in_Sd1_ED, ddu_in_Sd1_ED, p_out_Sd1_ED, p_in_Sd1_ED] = eval_dist_velocity_fields_first_ord_pseudo_analytical(Sd1_ED, x, c_Sd1_ED);

% Get velocity driven by Sd2(ED)
[u_out_Sd2_ED, du_out_Sd2_ED, u_in_Sd2_ED, du_in_Sd2_ED, ddu_in_Sd2_ED, p_out_Sd2_ED, p_in_Sd2_ED] = eval_dist_velocity_fields_second_ord_analytical(Sd2_ED, x, lambda, c_Sd2_ED);

% Get velocity driven by Sd3(ED)
[u_out_Sd3_ED, du_out_Sd3_ED, u_in_Sd3_ED, du_in_Sd3_ED, ddu_in_Sd3_ED, p_out_Sd3_ED, p_in_Sd3_ED] = eval_dist_velocity_fields_third_ord_pseudo_analytical(Sd3_ED, x, c_Sd3_ED);

% Get velocity driven by Sd4(ED)
[u_out_Sd4_ED, du_out_Sd4_ED, u_in_Sd4_ED, du_in_Sd4_ED, ddu_in_Sd4_ED, p_out_Sd4_ED, p_in_Sd4_ED] = eval_dist_velocity_fields_fourth_ord_analytical(Sd4_ED, x, lambda, c_Sd4_ED);


% Get velocity driven by D
[u_out_D, du_out_D, u_in_D, du_in_D, ddu_in_D, p_out_D, p_in_D] = eval_dist_velocity_fields_second_ord_analytical(D, x, lambda, c_D);

% Get velocity driven by D4
[u_out_D4, du_out_D4, u_in_D4, du_in_D4, ddu_in_D4, p_out_D4, p_in_D4] = eval_dist_velocity_fields_fourth_ord_analytical(D4, x, lambda, c_D4);

% Get velocity driven by Sd2(DD)
[u_out_Sd2_DD, du_out_Sd2_DD, u_in_Sd2_DD, du_in_Sd2_DD, ddu_in_Sd2_DD, p_out_Sd2_DD, p_in_Sd2_DD] = eval_dist_velocity_fields_second_ord_analytical(Sd2_DD, x, lambda, c_Sd2_DD);

% Get velocity driven by Sd4(DD)
[u_out_Sd4_DD, du_out_Sd4_DD, u_in_Sd4_DD, du_in_Sd4_DD, ddu_in_Sd4_DD, p_out_Sd4_DD, p_in_Sd4_DD] = eval_dist_velocity_fields_fourth_ord_analytical(Sd4_DD, x, lambda, c_Sd4_DD);
    

% Get total velocity and pressure fields
u_out      = E*x   + u_out_E     + u_out_D     + Ca * (u_out_Sd1_ED  +   u_out_Sd2_ED  +   u_out_Sd3_ED  +   u_out_Sd4_ED   +   u_out_Sd2_DD  +   u_out_Sd4_DD   +   u_out_D4  );
du_out     = E     + du_out_E    + du_out_D    + Ca * (du_out_Sd1_ED +   du_out_Sd2_ED +   du_out_Sd3_ED +   du_out_Sd4_ED  +   du_out_Sd2_DD +   du_out_Sd4_DD  +   du_out_D4 );
u_in       =         u_in_E      + u_in_D      + Ca * (u_in_Sd1_ED   +   u_in_Sd2_ED   +   u_in_Sd3_ED   +   u_in_Sd4_ED    +   u_in_Sd2_DD   +   u_in_Sd4_DD    +   u_in_D4   );
du_in      =         du_in_E     + du_in_D     + Ca * (du_in_Sd1_ED  +   du_in_Sd2_ED  +   du_in_Sd3_ED  +   du_in_Sd4_ED   +   du_in_Sd2_DD  +   du_in_Sd4_DD   +   du_in_D4  );
ddu_in     =         ddu_in_E    + ddu_in_D    + Ca * (ddu_in_Sd1_ED +   ddu_in_Sd2_ED +   ddu_in_Sd3_ED +   ddu_in_Sd4_ED  +   ddu_in_Sd2_DD +   ddu_in_Sd4_DD  +   ddu_in_D4 );
p_out      =         p_out_E     + p_out_D     + Ca * (p_out_Sd1_ED  +   p_out_Sd2_ED  +   p_out_Sd3_ED  +   p_out_Sd4_ED   +   p_out_Sd2_DD  +   p_out_Sd4_DD   +   p_out_D4  );
p_in       = -2*dH + p_in_E      + p_in_D      + Ca * (p_in_Sd1_ED   +   p_in_Sd2_ED   +   p_in_Sd3_ED   +   p_in_Sd4_ED    +   p_in_Sd2_DD   +   p_in_Sd4_DD    +   p_in_D4   );



% Evaluate jump in velocity and kinematic BC
u = u_out - u_in;
P = eye(3) - n * n';
P_u = P * u;
n_u = n' * u;
kin_BC = u_in' * grad_g;

% Evaluate jump in stress tensor
sigma_out = du_out + du_out' - p_out * eye(3);
sigma_in = lambda * (du_in + du_in') - p_in * eye(3);
P_sigma_n = P * ( (sigma_out - sigma_in) * n );
n_sigma_n = n' * ( (sigma_out - sigma_in) * n );

% Evaluate surface stress contributions
f_vis = eval_surface_vis(du_in, ddu_in, n, Dn);
f_dil = eval_surface_dil(du_in, ddu_in, n, Dn);

P_vis = -Bq_mu * P * f_vis;
P_dil = -Bq_k * P * f_dil;

n_vis = -Bq_mu * n' * f_vis;
n_dil = -Bq_k * n' * f_dil;

% Add surface stress contribution to surface tractions
P_sigma_n = P_sigma_n + P_vis + P_dil;
n_sigma_n = n_sigma_n + n_vis + n_dil;

end


% Evaluate surface viscosity contribution
% INPUT:  du = gradient of velocity field on surface
%         ddu = double gradient of velocity field on surface
%         n = normal vector
%         Dn = dn_i/dn_j = curvature tensor on surface

% OUTPUT:  f = surface traction due to surface shear viscosity
function f = eval_surface_vis(du, ddu, n, Dn)

% Note:  the expression for surface viscosity contribution is:
% fm = 2*H * n(g) * du(l, g) * P(l,m) + 2*H * n(l) * du(l, g) * P(m,g)
%        - P(n,g) * P(l,m) * ddu(l, g, n) 
%        + n(m) * du(l, g) * ( Dn(l,g) + Dn(g, l) - 2*H * P(g,l) )
% 
% where P = I - nn is the projection matrix,
%       du(i,j) = du_i/dx_j,   ddu(i,j,k) = du_i/dx_j dx_k
%       2*H = trace(Dn)


% First, get mean curvature and projection matrix
P = eye(3) - n * n';
H = 0.5 * trace(Dn);

% Evaluate fm
f = zeros(3,1);
for m = 1:3
    for g = 1:3
        for l = 1:3
            f(m) = f(m) + 2*H* du(l,g) * ( n(g) * P(m,l) + n(l) * P(m, g) );
            f(m) = f(m) + n(m) * du(l, g) * ( Dn(l,g) + Dn(g,l) - 2*H * P(g,l) );
        end
    end
end

for m = 1:3
    for g = 1:3
        for l = 1:3
            for n = 1:3
                f(m) = f(m) - P(n, g) * P(l, m) * ddu(l, g, n);
            end
        end
    end
end

end


% Evaluate dilatataional viscosity contribution
% INPUT:  du = gradient of velocity field on surface
%         ddu = double gradient of velocity field on surface
%         n = normal vector
%         Dn = dn_i/dn_j = curvature tensor on surface

% OUTPUT:  f = surface traction due to surface dilational viscosity
function f = eval_surface_dil(du, ddu, n, Dn)

% Note:  the expression for surface viscosity contribution is:
% fm = 2*H * n(m) * du(i, j) * P(i,j) + du(i, j) * ( Dn(i,m) * nj + Dn(j,m) * ni)
%        - P(n,m) * P(i,j) * ddu(i, j, n) 
% 
% where P = I - nn is the projection matrix,
%       du(i,j) = du_i/dx_j,   ddu(i,j,k) = du_i/dx_j dx_k
%       2*H = trace(Dn)


% First, get mean curvature and projection matrix
P = eye(3) - n * n';
H = 0.5 * trace(Dn);

% Evaluate fm
f = zeros(3,1);
for m = 1:3
    for i = 1:3
        for j = 1:3
            f(m) = f(m) + du(i, j) * ( 2*H*n(m) * P(i,j) + Dn(i, m)*n(j) + Dn(j,m) * n(i) );
        end
    end
end

for m = 1:3
    for i = 1:3
        for j = 1:3
            for n = 1:3
                f(m) = f(m) - P(n, m) * P(i, j) * ddu(i, j, n);
            end
        end
    end
end

end