% Evaulate disturbance velocity fields and pressure fields on deformed surface due to
% flow from first order pseduo tensor

% INPUT:  E = first order tensor
%         x = coordinate on surface
%         sol = coefficients of solution for O(1) velocity field

% Note:  the velocity fields satisfy:
%      u_i^out = A * eps_ijk * xj * Ek/r^3
%      p^out = 0

%      u_i^in = B * eps_ijk * xj * Ek
%      p^in = 0

% sol(1:4) is [A,B]

% OUTPUT:  u_out, du_out       = velocity field and gradients evaluated
%                                   on surface vector x.
%          u_in, du_in, ddu_in = velocity field and gradients evaluated on
%                                surface vector x
%          p_out, p_in = pressure fields evaluated at surface vector x

% Note:  du_in(i,j) = du_i/dx_j
%        ddu_in(i,j,k) = du_i / dx_j dx_k

function [u_out, du_out, u_in, du_in, ddu_in, ...
          p_out, p_in] = eval_dist_velocity_fields_first_ord_pseudo_analytical(E, x, sol)

% Rename coefs of solution
A = sol(1);
B = sol(2);

% Get Levi-Civita tensor
eps = zeros(3,3,3);
eps(1,2,3) = 1;       eps(2,3,1) = 1;        eps(3,1,2) = 1;
eps(1,3,2) = -1;      eps(2,1,3) = -1;       eps(3,2,1) = -1;

% Get outer velocity field and pressures
u_out = calc_u_out(E, x, A);
du_out = calc_du_out(E, x, eps, A);
p_out = 0;

% Get inner velocity field and pressures
u_in = calc_u_in(E, x, B);
du_in = calc_du_in(E, x, eps, B);
ddu_in = zeros(3,3,3);
p_in = 0;
    
end



% Anaytical calculations of pressure and velocity fields on surface
function u_out = calc_u_out(E, x, A)
    r = sqrt(x'*x);
    u_out = A * cross(x, E)/r^3;
end

function du_out = calc_du_out(E, x, eps, A)

% Intialize quantities
r = sqrt(x'*x);
du_out = zeros(3,3);

% Calculate gradient
for i = 1:3
    for q = 1:3
        for k = 1:3
            du_out(i,q) = du_out(i,q) + A * eps(i,q,k) * E(k)/r^3;
            for j = 1:3
                du_out(i,q) = du_out(i,q) - 3*A * x(q) * eps(i,j,k) * x(j) * E(k)/r^5;
            end
        end
    end
end

end


function u_in = calc_u_in(E, x, B)
    u_in = B * cross(x, E);
end

function du_in = calc_du_in(E, x, eps, B)

% Calculate gradient
du_in = zeros(3,3);
for i = 1:3
    for q = 1:3
        for k = 1:3
            du_in(i,q) = du_in(i,q) + B * eps(i,q,k) * E(k);
        end
    end
end

end

