% Evaulate disturbance velocity fields and pressure fields on deformed surface due to
% flow from fourth order tensor

% INPUT:  E = fourth order tensor
%         x = coordinate on surface
%         lambda = viscosity ratio
%         sol = coefficients of solution for O(1) velocity field

% Note:  the velocity fields satisfy:
%      u_i^out = Eijkl * xj * xk * xl (2A/r^7 + 4*B/r^9) + Emjkl * xm * xj * xk * xl * xi (-7A/r^9 - 9*B/r^11)
%      p^out = -14*A * Eijkl * xi * xj * xk * xl/r^9

%      u_i^in = Eijkl * xj * xk * xl (-28*C*r^2 + 4D) + Emjkl * xm * xj * xk * xl * xi (8*C)
%      p^in = -110*lambda*C * Emjkl * xm * xj * xk * xl

% sol(1:4) is [A,B,C,D]

% OUTPUT:  u_out, du_out, ddu_out = velocity field and gradients evaluated
%                                   on surface vector x.
%          u_in, du_in, ddu_in = velocity field and gradients evaluated on
%                                surface vector x
%          p_out, p_in = pressure fields evaluated at surface vector x

% Note:  du_out(i,j) = du_i/dx_j
%        ddu_out(i,j,k) = du_i / dx_j dx_k

% We will evaluate double derivatives using finite difference formulas.  In
% alternate function, we will evaluate this exactly

function [u_out, du_out, u_in, du_in, ddu_in, ...
          p_out, p_in] = eval_dist_velocity_fields_fourth_ord_analytical(E, x, lambda, sol)

% Rename coefs of solution
A = sol(1);
B = sol(2);
C = sol(3);
D = sol(4);

% Get outer velocity field and pressures
[u_out, p_out] = calc_u_out(E, x, A, B);
du_out = calc_du_out(E, x, A, B);

% Get inner velocity field and pressures
[u_in, p_in] = calc_u_in(E, x, C, D, lambda);
du_in = calc_du_in(E, x, C, D);
ddu_in = calc_ddu_in(E, x, C, D);
    
end



% Anaytical calculations of pressure and velocity fields on surface
function [u_out, p_out] = calc_u_out(E, x, A, B)

r = sqrt(x'*x);

% Get Exxx and x * Exxx
Exxx = zeros(3,1);
for i = 1:3
    for m = 1:3
        for j = 1:3
            for k = 1:3
                Exxx(i) = Exxx(i) + E(i,m,j,k) * x(m) * x(j) * x(k);
            end
        end
    end
end
xExxx = x' * Exxx;

u_out = Exxx * (2*A/r^7 + 4*B/r^9) + (xExxx) * x * (-7*A/r^9 - 9*B/r^11);
p_out = -14*A * xExxx/r^9;

end

function du_out = calc_du_out(E, x, A, B)

r = sqrt(x'*x);

% Get Exx, Exxx and x * Exxx
Exx = zeros(3,3);
for i = 1:3
    for j = 1:3
        for m = 1:3
            for k = 1:3
                Exx(i,j) = Exx(i,j) + E(i,j,m,k) * x(m) * x(k);
            end
        end
    end
end
Exxx = Exx * x;
xExxx = x' * Exxx;

du_out = 3 * Exx * (2*A/r^7 + 4*B/r^9) + Exxx * x' * (-14*A/r^9 - 36*B/r^11) ...
         + 4 * x * Exxx' * (-7*A/r^9 - 9*B/r^11) + eye(3) * xExxx * (-7*A/r^9 - 9*B/r^11) ...
         + xExxx * (x * x') * (63*A/r^11 + 99*B/r^13);

end


function [u_in, p_in] = calc_u_in(E, x, C, D, lambda)

r = sqrt(x'*x);

% Get Exxx and x * Exxx
Exxx = zeros(3,1);
for i = 1:3
    for m = 1:3
        for j = 1:3
            for k = 1:3
                Exxx(i) = Exxx(i) + E(i,m,j,k) * x(m) * x(j) * x(k);
            end
        end
    end
end
xExxx = x' * Exxx;

u_in = xExxx * x * (8*C) + Exxx * (4*D - 28*C*r^2);
p_in = -110*lambda*C * xExxx;

end

function du_in = calc_du_in(E, x, C, D)

r = sqrt(x'*x);

% Get Exx, Exxx and x * Exxx
Exx = zeros(3,3);
for i = 1:3
    for j = 1:3
        for m = 1:3
            for k = 1:3
                Exx(i,j) = Exx(i,j) + E(i,j,m,k) * x(m) * x(k);
            end
        end
    end
end
Exxx = Exx * x;
xExxx = x' * Exxx;

du_in = 32 * C * x * Exxx' + 8 * C * eye(3) * xExxx + Exx * (12*D - 28*3*C*r^2) + Exxx * x' * (-28*2*C);

end


function ddu_in = calc_ddu_in(E, x, C, D)

r = sqrt(x'*x);

% Get Ex, Exx, Exxx
Ex = zeros(3,3,3);
Exx = zeros(3,3);
for i = 1:3
    for j = 1:3
        for m = 1:3
            for k = 1:3
                Ex(i,j,m) = Ex(i,j,m) + E(i,j,m,k) * x(k);
                Exx(i,j) = Exx(i,j) + E(i,j,m,k) * x(m) * x(k);
            end
        end
    end
end
Exxx = Exx * x;

% Get ddu_in
ddu_in = zeros(3,3,3);
for i = 1:3
    for q = 1:3
        for g = 1:3
            ddu_in(i,q,g) = ddu_in(i,q,g) + 4*8*C * (i==q) * Exxx(g)...
                            + 32*3*C * Exx(q,g) * x(i) + 32*C * (i==g) * Exxx(q)...
                            + Ex(i,q,g) * (24*D - 28*3*2*C*r^2) + Exx(i,q) * x(g) * (-28*3*2*C)...
                            + Exx(i,g) * x(q) * (-28*3*2*C) + (q==g) * Exxx(i) * (-28*2*C);
        end
    end
end

end