% Evaulate disturbance velocity fields and pressure fields on deformed surface due to
% a second order tensor

% INPUT:  E = second order tensor
%         x = coordinate on surface
%         lambda = viscosity ratio
%         sol = coefficients of solution for O(1) velocity field

% Note:  the velocity fields satisfy:
%      u_i^out = Eij * xj * (2*B/r^5) + Ekj * xk * xj * xi (-3A/r^5 - 5*B/r^7)
%      p^out = -6*A * Eij * xi * xj/r^5

%      u_i^in = Eij * xj * (5*C*r^2 + D) + Ekj * xk * xj * xi * (-2*C)
%      p^in = 21*lambda*C * Eij * xi * xj

% sol(1:4) is [A,B,C,D]

% OUTPUT:  u_out, du_out, ddu_out = velocity field and gradients evaluated
%                                   on surface vector x.
%          u_in, du_in, ddu_in = velocity field and gradients evaluated on
%                                surface vector x
%          p_out, p_in = pressure fields evaluated at surface vector x

% Note:  du_out(i,j) = du_i/dx_j
%        ddu_out(i,j,k) = du_i / dx_j dx_k

% We will evaluate all derivatives analytically

function [u_out, du_out, u_in, du_in, ddu_in, ...
          p_out, p_in] = eval_dist_velocity_fields_second_ord_analytical(E, x, lambda, sol)

% Rename coefs of solution
A = sol(1);
B = sol(2);
C = sol(3);
D = sol(4);

% Get outer velocity field and pressures
[u_out, p_out] = calc_u_out(E, x, A, B);
du_out = calc_du_out(E, x, A, B);

% Get inner velocity field and pressures
[u_in, p_in] = calc_u_in(E, x, C, D, lambda);
du_in = calc_du_in(E, x, C, D);

% Evaluate ddu_in analytically
ddu_in = zeros(3,3,3);
Ex = E*x;
for i = 1:3
    for m = 1:3
        for l = 1:3
            ddu_in(i,m,l) = ddu_in(i,m,l) + E(i,m)*x(l) * 10*C + E(i,l)*x(m) * 10*C + Ex(i) * (m==l)*10*C ...
                            - 4*C * E(l,m)*x(i) - 4*C*Ex(m) * (i==l) - 4*C*Ex(l) * (i==m);
        end
    end
end
    
end



% Anaytical calculations of pressure and velocity fields on surface
function [u_out, p_out] = calc_u_out(E, x, A, B)

r = sqrt(x'*x);
Ex = E * x;
xEx = x' * Ex;
u_out = Ex * (2*B/r^5) + (xEx) * x * (-3*A/r^5 - 5*B/r^7);
p_out = -6*A * xEx/r^5;

end

function du_out = calc_du_out(E, x, A, B)

r = sqrt(x'*x);
Ex = E * x;
xEx = x' * Ex;
du_out = E * (2*B/r^5) + Ex * x' * -10*B/r^7 + x * Ex' * (-6*A/r^5 - 10*B/r^7) + xEx * eye(3) * (-3*A/r^5 - 5*B/r^7) + xEx * (x * x') * (15*A/r^7 + 35*B/r^9);

end


function [u_in, p_in] = calc_u_in(E, x, C, D, lambda)

r = sqrt(x'*x);
Ex = E * x;
xEx = x' * Ex;
u_in = Ex * (5*C*r^2 + D) - 2*C*x * xEx;
p_in = 21*lambda*C * xEx;

end

function du_in = calc_du_in(E, x, C, D)

r = sqrt(x'*x);
Ex = E * x;
xEx = x' * Ex;
du_in = E * (5*C*r^2 + D) + Ex * x' * 10 * C + x * Ex' * (-4*C) + xEx * eye(3) * (-2*C);

end

