% Evaulate disturbance velocity fields and pressure fields on deformed surface due to
% flow from third order pseduo tensor

% INPUT:  E = third order tensor
%         x = coordinate on surface
%         sol = coefficients of solution for O(1) velocity field

% Note:  the velocity fields satisfy:
%      u_i^out = A * eps_ijk * xj * Eklm * xl * xm/r^7
%      p^out = 0

%      u_i^in = B * eps_ijk * xj * Eklm * xl * xm
%      p^in = 0

% sol(1:4) is [A,B]

% OUTPUT:  u_out, du_out       = velocity field and gradients evaluated
%                                   on surface vector x.
%          u_in, du_in, ddu_in = velocity field and gradients evaluated on
%                                surface vector x
%          p_out, p_in = pressure fields evaluated at surface vector x

% Note:  du_in(i,j) = du_i/dx_j
%        ddu_in(i,j,k) = du_i / dx_j dx_k

function [u_out, du_out, u_in, du_in, ddu_in, ...
          p_out, p_in] = eval_dist_velocity_fields_third_ord_pseudo_analytical(E, x, sol)

% Rename coefs of solution
A = sol(1);
B = sol(2);

% Get Levi-Civita tensor
eps = zeros(3,3,3);
eps(1,2,3) = 1;       eps(2,3,1) = 1;        eps(3,1,2) = 1;
eps(1,3,2) = -1;      eps(2,1,3) = -1;       eps(3,2,1) = -1;

% Get outer velocity field and pressures
u_out = calc_u_out(E, x, A);
du_out = calc_du_out(E, x, eps, A);
p_out = 0;

% Get inner velocity field and pressures
u_in = calc_u_in(E, x, B);
du_in = calc_du_in(E, x, eps, B);
ddu_in = calc_ddu_in(E, x, eps, B);
p_in = 0;
    
end



% Anaytical calculations of pressure and velocity fields on surface
function u_out = calc_u_out(E, x, A)

r = sqrt(x'*x);
    
% Calculate Exx
Exx = zeros(3,1);
for i = 1:3
    for j = 1:3
        for k = 1:3
            Exx(i) = Exx(i) + E(i,j,k) * x(j) * x(k);
        end
    end
end
u_out = A * cross(x, Exx)/r^7;

end

function du_out = calc_du_out(E, x, eps, A)

% Intialize quantities
r = sqrt(x'*x);
du_out = zeros(3,3);

% Calculate Ex and Exx
Ex = zeros(3,3);
Exx = zeros(3,1);
for i = 1:3
    for j = 1:3
        for k = 1:3
            Ex(i,j) = Ex(i,j) + E(i,j,k) * x(k);
            Exx(i) = Exx(i) + E(i,j,k) * x(j) * x(k);
        end
    end
end


% Calculate gradient
for i = 1:3
    for q = 1:3
        for k = 1:3
            du_out(i,q) = du_out(i,q) + A * eps(i,q,k) * Exx(k)/r^7;
            for j = 1:3
                du_out(i,q) = du_out(i,q) + 2*A * eps(i,j,k) * x(j) * Ex(k,q)/r^7 - 7*A * x(q) * eps(i,j,k) * x(j) * Exx(k)/r^9;
            end
        end
    end
end

end


function u_in = calc_u_in(E, x, B)

% Calculate Exx
Exx = zeros(3,1);
for i = 1:3
    for j = 1:3
        for k = 1:3
            Exx(i) = Exx(i) + E(i,j,k) * x(j) * x(k);
        end
    end
end

% Calculate u_in
u_in = B * cross(x, Exx);

end

function du_in = calc_du_in(E, x, eps, B)

% Intialize quantities
du_in = zeros(3,3);

% Calculate Ex and Exx
Ex = zeros(3,3);
Exx = zeros(3,1);
for i = 1:3
    for j = 1:3
        for k = 1:3
            Ex(i,j) = Ex(i,j) + E(i,j,k) * x(k);
            Exx(i) = Exx(i) + E(i,j,k) * x(j) * x(k);
        end
    end
end


% Calculate gradient
for i = 1:3
    for q = 1:3
        for k = 1:3
            du_in(i,q) = du_in(i,q) + B * eps(i,q,k) * Exx(k);
            for j = 1:3
                du_in(i,q) = du_in(i,q) + 2*B * eps(i,j,k) * x(j) * Ex(k,q);
            end
        end
    end
end

end


function ddu_in = calc_ddu_in(E, x, eps, B)

% Intialize quantities
ddu_in = zeros(3,3,3);

% Calculate Ex and Exx
Ex = zeros(3,3);
Exx = zeros(3,1);
for i = 1:3
    for j = 1:3
        for k = 1:3
            Ex(i,j) = Ex(i,j) + E(i,j,k) * x(k);
            Exx(i) = Exx(i) + E(i,j,k) * x(j) * x(k);
        end
    end
end


% Calculate gradient
for i = 1:3
    for q = 1:3
        for k = 1:3
            for p = 1:3
                ddu_in(i,q,p) = ddu_in(i,q,p) + 2*B*eps(i,q,k) * Ex(k,p) + 2*B * eps(i,p,k) * E(k,q);
                for j = 1:3
                    ddu_in(i,q,p) = ddu_in(i,q,p) + 2*B * eps(i,j,k) * x(j) * E(k,q,p);
                end
            end
        end
    end
end

end