% calc_coefs_ord_Ca_sq
% This function calculates the coefficients of the velocity field at
% O(Ca^2) that is driven by second order harmonics

% INPUT:  lambda = viscosity ratio
%         Bq_mu = shear Boussinesq number
%         Bq_k = dilational Boussinesq number

% OUTPUT:  coefs_EDD  = [A, B, C, D, a] for flow driven by Sd2[E*D*D]
%          coefs_E_DD = [A, B, C, D, a] for flow driven by E (D**D)
%          coefs_D_DE = [A, B, C, D, a] for flow driven by D (D**E)
%          coefs_ED4  = [A, B, C, D, a] for flow driven by E**D4
%          coefs_D_DD = [A, B, C, D, a] for flow driven by D (D**D)
%          coefs_D_D4 = [A, B, C, D, a] for flow driven by D ** D4

% The last index of each of the coefs (i.e., a) is the coefficient for the
% shape evolution equation for the tensors at O(Ca^2)


function [coefs_EDD, coefs_E_DD, coefs_D_DE, coefs_ED4, coefs_D_DD, coefs_D_D4] = calc_coefs_ord_Ca_sq(lambda, Bq_mu, Bq_k)

% Matrix for converting BCs into coefs [A, B, C, D, a].  Rows correspond to
% (a) kinematic BC, (b) tangential velocity, (c) normal velocity, (d)
% tangential stress, (e) normal stress
M =   [           0,   0,                               3,                             1,     -1;
                  0,   2,                              -5,                            -1,     0;
                 -3,  -3,                              -3,                            -1,     0;
                 -6, -16, -16*lambda - 18*Bq_k - 20*Bq_mu,  -2*lambda - 2*Bq_k - 4*Bq_mu,     0;
                 18,  24,              3*lambda + 18*Bq_k,            -2*lambda + 2*Bq_k,     0];



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% Get ED4 component %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Strategy:  We will let D = 0, E = uniaxial flow, and D4 = Sd4[E,E].  If we
% Taylor expand the boundary conditions to O(Ca^2) and project onto second order harmonics,
% only E ** D4 will survive.  Thus, we can get coefs for ED4

E = [-0.5,    0, 0;
        0, -0.5, 0;
        0,    0, 1];
    
D = zeros(3,3);
D4 = Sd4(E,E);
ED4 = zeros(3,3);
for i = 1:3
    for j = 1:3
        for k = 1:3
            for m = 1:3
                ED4(i,j) = ED4(i,j) + D4(i,j,k,m) * E(k,m);
            end
        end
    end
end


% Calculate O(1) and O(Ca) fields and Taylor expand the boundary conditions to O(Ca^2).  Then, project result onto second order harmonics
flag_proj = 2;
flag_ord_Ca = 2;
comp = calc_projections_updated(E, D, D4, lambda, Bq_mu, Bq_k, flag_proj, flag_ord_Ca);
b_ED4 = comp(5,:)'/ED4(3,3);     % Only (3,3) components will be nonzero for each boundary condition.  Divide by ED4(3,3) to get BCs in units of this tensor
coefs_ED4 = M \ b_ED4;           % Solve for [A, B, C, D, a] for flow


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% Get D (D**D) component %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Strategy:  We will let E = 0, D4 = 0, and D = uniaxial flow.  If we
% Taylor expand the boundary conditions to O(Ca^2) and project onto second order harmonics,
% only Sd2[D*D*D] will survive.  Thus, we can get coefs for this term

E = zeros(3,3);
D = [-0.5,    0, 0;
        0, -0.5, 0;
        0,    0, 1];
    
D_DD = D * trace(D*D);
D4 = zeros(3,3,3,3);

% Calculate O(1) and O(Ca) fields and Taylor expand BCs to O(Ca^2).  Then, project result onto second order harmonics
flag_proj = 2;
flag_ord_Ca = 2;
comp = calc_projections_updated(E, D, D4, lambda, Bq_mu, Bq_k, flag_proj, flag_ord_Ca);
b_D_DD = comp(5,:)'/D_DD(3,3);      % Only (3,3) components will be nonzero for each boundary condition.  Divide by D_DD(3,3) to get BCs in units of this tensor
coefs_D_DD = M \ b_D_DD;            % Solve for [A, B, C, D, a] for flow


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% Get D4 ** D component %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Strategy:  We will let E = 0, D = uniaxial flow, and D4 = Sd4[D,D].  If we
% Taylor expand the boundary conditions to O(Ca^2) and project onto second order harmonics,
% terms with Sd2[D*D*D] and D4 ** D will survive.  We already know the contribution
% from Sd2[D*D*D], so we can subract this off, and hence get coefs for
% D4**D

E = zeros(3,3);
D = [-0.5,    0, 0;
        0, -0.5, 0;
        0,    0, 1];
    
D_DD = D * trace(D*D);
D4 = Sd4(D,D);
DD4 = zeros(3,3);
for i = 1:3
    for j = 1:3
        for k = 1:3
            for m = 1:3
                DD4(i,j) = DD4(i,j) + D4(i,j,k,m) * D(k,m);
            end
        end
    end
end

% Calculate O(1) and O(Ca) fields and Taylor expand BCs to O(Ca^2).  Then, project result onto second order harmonics
flag_proj = 2;
flag_ord_Ca = 2;
comp = calc_projections_updated(E, D, D4, lambda, Bq_mu, Bq_k, flag_proj, flag_ord_Ca);
b_D_D4 = (comp(5,:)' - b_D_DD*D_DD(3,3))/DD4(3,3);      % Only (3,3) components survive.  Subtract off contribution from D_DD and divide by D_D4(3,3) to get contribution in units of D_D4
coefs_D_D4 = M \ b_D_D4;                                % Solve for [A, B, C, D, a] for this flow



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Get E*D*D and  E (D**D) components %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Strategy:  We will let E = extensional flow (12), D = extensional flow (13 and 23), and D4 = 0.  If we
% project the boundary conditions to O(Ca^2) and project onto second order harmonics,
% terms with Sd2[E*D*D], E(D**D), and D(D**D) will survive.  The compoent in the (33) direction will consist 
% only of Sd2[E*D*D], while the the component in the (12) direction will
% consist of E(D**D) and Sd2[E*D*D]

E = [0,  1,    0;
     1,  0,    0;
     0,  0,    0];
 
D = [0, 0,  1;
     0, 0,  1;
     1, 1,  0];
 

D4 = zeros(3,3,3,3);
EDD = Sd2(E*D*D);
E_DD = E * trace(D*D);
D_DE = D * trace(D*E);
D_DD = D * trace(D*D);
 
% Calculate O(1) and O(Ca) fields and Taylor expand BCs to O(Ca^2).  Then, project result onto second order harmonics
flag_proj = 2;
flag_ord_Ca = 2;
comp = calc_projections_updated(E, D, D4, lambda, Bq_mu, Bq_k, flag_proj, flag_ord_Ca);
comp(3,:) = comp(3,:) - b_D_DD' * D_DD(1,3);              % Subtract off D_DD component in (1,3) and (2,3) directions
comp(4,:) = comp(4,:) - b_D_DD' * D_DD(2,3);

% Get EDD and E_DD component
b_EDD = comp(5,:)'/EDD(3,3);                              % Normalize by EDD(3,3) to get in units of this tensor
b_E_DD = ( comp(2,:)' - b_EDD * EDD(1,2) )/E_DD(1,2);     % Subtract off EDD term and normalize in terms of E(D**D)
coefs_EDD = M \ b_EDD;                                    % Get coefs [A, B, C, D, a] for Sd2[E*D*D]
coefs_E_DD = M \ b_E_DD;                                  % Get coefs [A, B, C, D, a] for E (D**D)





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Get D (D**E) components %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Strategy:  We will let E = D = uniaxial extensional flow and D4 = 0.  If we
% project the boundary conditions to O(Ca^2) and project onto second order harmonics,
% terms with Sd2[E*D*D], E(D**D), D(D**E), and D(D**D) will survive.  If we subtract
% the D (D**D), Sd2[E*D*D], and E(D**D) components, the compoent in the (3,3) direction will consist 
% only of D (D**E).  


E = [-1/2,     0,    0;
     0   ,  -1/2,    0;
     0   ,     0,    1];
 
D = [-1/2,     0,    0;
     0   ,  -1/2,    0;
     0   ,     0,    1];
 

D4 = zeros(3,3,3,3);
EDD = Sd2(E*D*D);
E_DD = E * trace(D*D);
D_DE = D * trace(D*E);
D_DD = D * trace(D*D);
 
% Calculate O(1) and O(Ca) fields and Taylor expand BCs to O(Ca^2).  Then, project result onto second order harmonics
flag_proj = 2;
flag_ord_Ca = 2;
comp = calc_projections_updated(E, D, D4, lambda, Bq_mu, Bq_k, flag_proj, flag_ord_Ca);

% Get D_DE component
b_D_DE = ( comp(5,:)' - b_D_DD * D_DD(3,3) - b_E_DD * E_DD(3,3) - b_EDD * EDD(3,3) )/D_DE(3,3);    % Subtract D (D**D), E (D**D), and Sd2[E*D*D] components and normalize by D_DE(3,3)
coefs_D_DE = M \ b_D_DE;                        % Get coefs [A, B, C, D, a] for this flow


end