% Project boundary conditions driven by disturbance velocity field onto spherical harmonics
% INPUT:  E = external strain rate tensor
%         D = 2nd order deformation matrix
%         D4 = 4th order deformation matrix
%         lambda = viscosity ratio
%         Bq_mu, Bq_k = shear and dilational interfacial viscosity
%         flag_proj = flag for harmonics we project BCs.  flag_proj = 2 if we project onto 2nd order harmonics, flag_proj = 4 if
%                     we project onto fourth order harmonics
%         flag_ord_Ca = order of perturbation (1 = O(Ca), 2 = O(Ca^2))

% OUTPUT:
% If projecting onto second order harmonics,
% comp(:, 1) = components of kinematic BC
% comp(:, 2) = tangential component of [[ui]]
% comp(:, 3) = normal component of [[ui]]
% comp(:, 4) = tangential part of stress
% comp(:, 5) = normal part of stress
% rows are [E11 - E22, E12, E13, E23, E33] components of 2nd order tensor 
% rows are (40, 41, 4-1, 42, 4-2, 43, 4-3, 44, 4-4) components for 4th order tensor.  We add
% factor 1/5 for convenience, which helps in determining weights of various
% 4th order tensors

function comp = calc_projections_updated(E, D, D4, lambda, Bq_mu, Bq_k, flag_proj, flag_ord_Ca)

% Zero projection of velocity and stress components onto spherical
% harmonics
if( flag_proj == 2 )    % 2nd order harmonics
    u_tang_proj = zeros(5,1);
    u_n_proj = zeros(5,1);
    kin_proj = zeros(5,1);
    stress_tang_proj = zeros(5,1);
    stress_norm_proj = zeros(5,1);
else                   % 4th order harmonics
    u_tang_proj = zeros(9,1);
    u_n_proj = zeros(9,1);
    kin_proj = zeros(9,1);
    stress_tang_proj = zeros(9,1);
    stress_norm_proj = zeros(9,1);
end


% For each value of theta and phi, calculate velocity field
N_theta = 10;
N_phi = 10;
[nodes, weights] = get_Gaussian_quad_weights(N_theta);
theta = acos(nodes);
phi = 2*pi/(2*N_phi) * (1:2:(2*N_phi-1));
d_phi = 2*pi/N_phi;

for i = 1:length(theta)
    for j = 1:length(phi)
        
        
        % For value of theta and phi, calculate boundary conditions on
        % deformed surface to appropriate order of Ca.  We will perform
        % finite difference schemes to evaluate derivatives
        %Ca = 5e-4;
        Ca = 1e-3;
        if( flag_ord_Ca == 1 )    % evaluate O(Ca) contribution
            
            [P_u_right, n_u_right, kin_right, P_sigma_n_right, n_sigma_n_right] = ...
                                                   eval_BCs_updated_v2(E, D, D4, theta(i), phi(j), 0.5*Ca, lambda, Bq_mu, Bq_k);
                                                                            
            [P_u_left, n_u_left, kin_left, P_sigma_n_left, n_sigma_n_left] = ...
                                                   eval_BCs_updated_v2(E, D, D4, theta(i), phi(j), -0.5*Ca, lambda, Bq_mu, Bq_k);            
            
            % Finite differences
            P_u = (P_u_right - P_u_left)/Ca;
            n_u = (n_u_right - n_u_left)/Ca;
            kin = (kin_right - kin_left)/Ca;
            P_sigma_n = (P_sigma_n_right - P_sigma_n_left)/Ca;
            n_sigma_n = (n_sigma_n_right - n_sigma_n_left)/Ca;
            
        else      % evaluate O(Ca^2) contribution
            
            [P_u_right, n_u_right, kin_right, P_sigma_n_right, n_sigma_n_right] = ...
                                                   eval_BCs_updated_v2(E, D, D4, theta(i), phi(j), Ca, lambda, Bq_mu, Bq_k);
                                                                            
            [P_u_cen, n_u_cen, kin_cen, P_sigma_n_cen, n_sigma_n_cen] = ...
                                                   eval_BCs_updated_v2(E, D, D4, theta(i), phi(j), 0, lambda, Bq_mu, Bq_k);
            
            [P_u_left, n_u_left, kin_left, P_sigma_n_left, n_sigma_n_left] = ...
                                                   eval_BCs_updated_v2(E, D, D4, theta(i), phi(j), -Ca, lambda, Bq_mu, Bq_k);
            
            
            % Finite differences
            P_u = 0.5 * (P_u_right - 2 * P_u_cen + P_u_left)/Ca^2;
            n_u = 0.5 * (n_u_right - 2 * n_u_cen + n_u_left)/Ca^2;
            kin = 0.5 * (kin_right - 2 * kin_cen + kin_left)/Ca^2;
            P_sigma_n = 0.5 * (P_sigma_n_right - 2 * P_sigma_n_cen + P_sigma_n_left)/Ca^2;
            n_sigma_n = 0.5 * (n_sigma_n_right - 2 * n_sigma_n_cen + n_sigma_n_left)/Ca^2;

        end
        
        
        
        % Project boundary conditions onto spherical harmonics
        if( flag_proj == 2 )   % 2nd order harmonics
            [Ylm, Psi_lm] = get_2nd_ord_harmonics(theta(i), phi(j));
        else
            [Ylm, Psi_lm] = get_4th_ord_harmonics(theta(i), phi(j));
        end
        u_tang_proj = u_tang_proj + weights(i) * d_phi * (Psi_lm * P_u);
        u_n_proj = u_n_proj + weights(i) * d_phi * n_u * Ylm;
        kin_proj = kin_proj + weights(i) * d_phi * kin * Ylm;
        stress_tang_proj = stress_tang_proj + weights(i) * d_phi * (Psi_lm * P_sigma_n);
        stress_norm_proj = stress_norm_proj + weights(i) * d_phi * n_sigma_n * Ylm;
        
    end
end

% Get rhs
if( flag_proj == 2)  % projecting onto 2nd order harmonics
    rhs = zeros(5, 5);
else
    rhs = zeros(9, 5);
end
rhs(:, 1) = -kin_proj;
rhs(:, 2) = -u_tang_proj;
rhs(:, 3) = -u_n_proj;
rhs(:, 4) = -stress_tang_proj;
rhs(:, 5) = -stress_norm_proj;

% Convert into tensor components
if( flag_proj == 2)   % Projecting onto 2nd order harmonics
    comp = convert_harmonic_modes_to_tensor_coefs(rhs);
else                  % Projecting onto 4th order harmonics
    comp = rhs;
    comp(:,2) = 1/5 * rhs(:,2);       % Multiply projection by 1/5 for tangential boundary conditions
    comp(:,4) = 1/5 * rhs(:,4);
end

end




% function: convert_harmonic_modes_to_tensor_coefs
% This function converts the projected boundary condition into 2nd order
% tensor components

% INPUT:  proj = projection of BCs onto [20, 21, 2-1, 22, 2-2] modes
%                rows 1-5 = [20, 21, 2-1, 22, 2-2] modes
%                column_1 = kinematic BC
%                column_2 = P*[[u]]
%                column_3 = n * [[u]]
%                column_4 = P * [[sigma * n]]
%                column_5 = n * [[sigma * n]]

% OUTPUT:  comp = [E11 - E22, E12, E13, E23, E33] components of 2nd order
%                 tensor.  Rows are the components, columns correspond to
%                 the boundary conditions listed above


function comp = convert_harmonic_modes_to_tensor_coefs(proj)

comp = zeros(size(proj));
for i = 1:5
    if(i == 1 || i == 3 || i == 5 )  % Scalar vs. vector harmonics
        flag_scalar = 1;
    else
        flag_scalar = 0;
    end
    comp(:, i) = get_tensor_comp(proj(:, i), flag_scalar);   % convert projected BC into tensor components
end

end





% Get tensor components given projection of boundary condition onto 2nd
% order harmonics

% INPUT:  proj = projection of BC onto [20, 21, 2-1, 22, 2-2] modes
%         flag_scalar = flag whether we are using scalar harmonics or
%                       vector harmonics

% OUTPUT:  comp = [E11 - E22, E12, E13, E23, E33] components of 2nd order
%                 tensor

% Math:  If we have boundary condition equalling
%        BC = proj(1) * Y20 + proj(2) * Y21 + proj(3) * Y2-1 + proj(4) * Y22 + proj(5) * Y2-2
%       
%        We want to rexpress this into BC = Eij * ri * rj, where Eij is a 2nd order tensor
%        We get the components of Eij here

%        If the boundary condition equals
%        BC = proj(1) * Psi20 + proj(2) * Psi21 + proj(3) * Psi2-1 + proj(4) * Psi22 + proj(5) * Psi2-2
%        where Psi_lm is the gradient of the spherical harmonics, then we
%        can rexpress the BC into:
%        BC = (delta_ik - ri * rk) * Ekj * rj.  We solve for Eij here

function comp = get_tensor_comp(proj, flag_scalar)

comp = zeros(5,1);

% Get E11-E22
comp(1) = 1/2 * sqrt(15/pi) * proj(4);

% Get E12
comp(2) = 1/4 * sqrt(15/pi) * proj(5);

% Get E13
comp(3) = -1/4 * sqrt(15/pi) * proj(2);

% Get E23
comp(4) = -1/4 * sqrt(15/pi) * proj(3);

% Get E33
comp(5) = 1/2 * sqrt(5/pi) * proj(1);


% Rescale if using vector harmonics
if( flag_scalar == 0 )
    comp = 1/3 * comp;
end

end