% function: convert_harmonic_modes_to_tensor_coefs
% This function converts the projected boundary condition into 2nd order
% tensor components

% INPUT:  proj = projection of BCs onto [20, 21, 2-1, 22, 2-2] modes
%                rows 1-5 = [20, 21, 2-1, 22, 2-2] modes
%                column_1 = kinematic BC
%                column_2 = P*[[u]]
%                column_3 = n * [[u]]
%                column_4 = P * [[sigma * n]]
%                column_5 = P * f_vis
%                column_6 = P * f_dil
%                column_7 = n * [[sigma * n]]
%                column_8 = n * f_vis
%                column_9 = n * f_dil


% OUTPUT:  comp = [E11 - E22, E12, E13, E23, E33] components of 2nd order
%                 tensor.  Rows are the components, columns correspond to
%                 the boundary conditions listed above


function comp = convert_harmonic_modes_to_tensor_coefs(proj)

comp = zeros(size(proj));
for i = 1:9
    if(i == 1 || i == 3 || i == 7 || i == 8 || i == 9 )  % Scalar vs. vector harmonics
        flag_scalar = 1;
    else
        flag_scalar = 0;
    end
    comp(:, i) = get_tensor_comp(proj(:, i), flag_scalar);   % convert projected BC into tensor components
end

end





% Get tensor components given projection of boundary condition onto 2nd
% order harmonics

% INPUT:  proj = projection of BC onto [20, 21, 2-1, 22, 2-2] modes
%         flag_scalar = flag whether we are using scalar harmonics or
%                       vector harmonics

% OUTPUT:  comp = [E11 - E22, E12, E13, E23, E33] components of 2nd order
%                 tensor

% Math:  If we have boundary condition equalling
%        BC = proj(1) * Y20 + proj(2) * Y21 + proj(3) * Y2-1 + proj(4) * Y22 + proj(5) * Y2-2
%       
%        We want to rexpress this into BC = Eij * ri * rj, where Eij is a 2nd order tensor
%        We get the components of Eij here

%        If the boundary condition equals
%        BC = proj(1) * Psi20 + proj(2) * Psi21 + proj(3) * Psi2-1 + proj(4) * Psi22 + proj(5) * Psi2-2
%        where Psi_lm is the gradient of the spherical harmonics, then we
%        can rexpress the BC into:
%        BC = (delta_ik - ri * rk) * Ekj * rj.  We solve for Eij here

function comp = get_tensor_comp(proj, flag_scalar)

comp = zeros(5,1);

% Get E11-E22
comp(1) = 1/2 * sqrt(15/pi) * proj(4);

% Get E12
comp(2) = 1/4 * sqrt(15/pi) * proj(5);

% Get E13
comp(3) = -1/4 * sqrt(15/pi) * proj(2);

% Get E23
comp(4) = -1/4 * sqrt(15/pi) * proj(3);

% Get E33
comp(5) = 1/2 * sqrt(5/pi) * proj(1);


% Rescale if using vector harmonics
if( flag_scalar == 0 )
    comp = 1/3 * comp;
end

end