% Evaulate boundary conditions on deformed surface due to disturbance flow

% INPUT:  L = 2nd order tensor
%         x = coordinate on surface
%         n = normal vector on surface
%         grad_g = grad( r - rs ) on surface for kinematic BC
%         Dn = dn_i/dn_j = curvature tensor on surface
%         lambda = viscosity ratio
%         Bq_mu = Boussinesq number for shear viscosity
%         Bq_k = Boussinesq number for dilatational viscosity
%         sol = coefficients of solution for O(1) velocity field

% Note:  If L = second order tensor, velocity fields satisfy:
%      u_i^out = Lij * xj * (1 + 2*B/r^5) + Lkj * xk * xj * xi (-3A/r^5 - 5*B/r^7)
%      p^out = -6*A * Lij * xi * xj/r^5

%      u_i^in = Lij * xj * (5*C*r^2 + D) + Lkj * xk * xj * xi * (-2*C)
%      p^in = 21*lambda*C * Lij * xi * xj

% sol(1:4) is [A,B,C,D]


% OUTPUT:  u = (uout - uin) = jump in disturbance velocity field across deformed surface
%          sigma = [sigma_out*n - sigma_in*n] =
%                      jump in disturbance traction across deformed surface
%          fvis = -Bq_mu * f_mu = disturbance traction due to surface shear viscosity
%          fdil = -Bq_k * f_k = disturbance traction due to surface dilatational viscosity

% Note:  du_out(i,j) = du_i/dx_j
%        ddu_out(i,j,k) = du_i / dx_j dx_k

function [u, traction, f_vis, f_dil] = eval_BCs_pseudo(L, x, n, grad_g, Dn, lambda, Bq_mu, Bq_k, sol)

% Evaluate velocity and pressure fields on surface
[u_out, du_out, u_in, du_in, ddu_in, p_out, p_in] = eval_dist_velocity_fields_second_ord_analytical(L, x, lambda, sol);

% Evaluate jump in velocity and stress tensor
u = u_out - u_in;

% Evaluate jump in stress tensor
sigma_out = du_out + du_out' - p_out * eye(3);
sigma_in = lambda * (du_in + du_in') - p_in * eye(3);
traction = ( sigma_out - sigma_in ) * n;

% Evaluate surface stress contributions
f_vis = eval_surface_vis(du_in, ddu_in, n, Dn);
f_dil = eval_surface_dil(du_in, ddu_in, n, Dn);
f_vis = -Bq_mu * f_vis;
f_dil = -Bq_k * f_dil;

end


% Evaluate surface viscosity contribution
% INPUT:  du = gradient of velocity field on surface
%         ddu = double gradient of velocity field on surface
%         n = normal vector
%         Dn = dn_i/dn_j = curvature tensor on surface

% OUTPUT:  f = surface traction due to surface shear viscosity
function f = eval_surface_vis(du, ddu, n, Dn)

% Note:  the expression for surface viscosity contribution is:
% fm = 2*H * n(g) * du(l, g) * P(l,m) + 2*H * n(l) * du(l, g) * P(m,g)
%        - P(n,g) * P(l,m) * ddu(l, g, n) 
%        + n(m) * du(l, g) * ( Dn(l,g) + Dn(g, l) - 2*H * P(g,l) )
% 
% where P = I - nn is the projection matrix,
%       du(i,j) = du_i/dx_j,   ddu(i,j,k) = du_i/dx_j dx_k
%       2*H = trace(Dn)


% First, get mean curvature and projection matrix
P = eye(3) - n * n';
H = 0.5 * trace(Dn);

% Evaluate fm
f = zeros(3,1);
for m = 1:3
    for g = 1:3
        for l = 1:3
            f(m) = f(m) + 2*H* du(l,g) * ( n(g) * P(m,l) + n(l) * P(m, g) );
            f(m) = f(m) + n(m) * du(l, g) * ( Dn(l,g) + Dn(g,l) - 2*H * P(g,l) );
        end
    end
end

for m = 1:3
    for g = 1:3
        for l = 1:3
            for n = 1:3
                f(m) = f(m) - P(n, g) * P(l, m) * ddu(l, g, n);
            end
        end
    end
end

end


% Evaluate dilatataional viscosity contribution
% INPUT:  du = gradient of velocity field on surface
%         ddu = double gradient of velocity field on surface
%         n = normal vector
%         Dn = dn_i/dn_j = curvature tensor on surface

% OUTPUT:  f = surface traction due to surface dilational viscosity
function f = eval_surface_dil(du, ddu, n, Dn)

% Note:  the expression for surface viscosity contribution is:
% fm = 2*H * n(m) * du(i, j) * P(i,j) + du(i, j) * ( Dn(i,m) * nj + Dn(j,m) * ni)
%        - P(n,m) * P(i,j) * ddu(i, j, n) 
% 
% where P = I - nn is the projection matrix,
%       du(i,j) = du_i/dx_j,   ddu(i,j,k) = du_i/dx_j dx_k
%       2*H = trace(Dn)


% First, get mean curvature and projection matrix
P = eye(3) - n * n';
H = 0.5 * trace(Dn);

% Evaluate fm
f = zeros(3,1);
for m = 1:3
    for i = 1:3
        for j = 1:3
            f(m) = f(m) + du(i, j) * ( 2*H*n(m) * P(i,j) + Dn(i, m)*n(j) + Dn(j,m) * n(i) );
        end
    end
end

for m = 1:3
    for i = 1:3
        for j = 1:3
            for n = 1:3
                f(m) = f(m) - P(n, m) * P(i, j) * ddu(i, j, n);
            end
        end
    end
end

end