% function: convert_harmonic_modes_to_tensor_coefs_first_ord
% This function converts the projected boundary condition into 1st order
% tensor components

% INPUT:  proj = projection of BCs onto [10, 11, 1-1] modes
%                rows 1-5 = [10, 11, 1-1] modes
%                column_1 = kinematic BC
%                column_2 = P*[[u]]
%                column_3 = n * [[u]]
%                column_4 = P * [[sigma * n]]
%                column_5 = P * f_vis
%                column_6 = P * f_dil
%                column_7 = n * [[sigma * n]]
%                column_8 = n * f_vis
%                column_9 = n * f_dil


% OUTPUT:  comp = [E1, E2, E3] components of 1st order
%                 tensor.  Rows are the components, columns correspond to
%                 the boundary conditions listed above


function comp = convert_harmonic_modes_to_tensor_coefs_first_ord(proj, flag_pseudo)

comp = zeros(size(proj));

if( flag_pseudo == 0 )
    for i = 1:9
        if(i == 1 || i == 3 || i == 7 || i == 8 || i == 9 )  % Scalar vs. vector harmonics
            flag_scalar = 1;
        else
            flag_scalar = 0;
        end
        comp(:, i) = get_tensor_comp(proj(:, i), flag_scalar);   % convert projected BC into tensor components
    end
    
else 
    flag_scalar = 0;
    for i = 1:4
        comp(:,i) = get_tensor_comp(proj(:,i), flag_scalar);
    end
end

end





% Get tensor components given projection of boundary condition onto 2nd
% order harmonics

% INPUT:  proj = projection of BC onto [10, 11, 1-1] modes
%         flag_scalar = flag whether we are using scalar harmonics or
%                       vector harmonics

% OUTPUT:  comp = [E1, E2, E3] components of 1st order
%                 tensor

% Math:  If we have boundary condition equalling
%        BC = proj(1) * Y10 + proj(2) * Y11 + proj(3) * Y1-1
%       
%        We want to rexpress this into BC = Ei * ri, where Ei is a 1st order tensor
%        We get the components of Ei here

%        If the boundary condition equals
%        BC = proj(1) * Psi10 + proj(2) * Psi11 + proj(3) * Psi1-1
%        where Psi_lm is the gradient of the spherical harmonics, then we
%        can rexpress the BC into:
%        BC = (delta_ik - ri * rk) * Ek.  We solve for Ek here

function comp = get_tensor_comp(proj, flag_scalar)

comp = zeros(3,1);

% Get E3
comp(3) = sqrt(3/(4*pi)) * proj(1);

% Get E1
comp(1) = -sqrt(3/(4*pi)) * proj(2);

% Get E2
comp(2) = -sqrt(3/(4*pi)) * proj(3);

% Rescale if using vector harmonics
if( flag_scalar == 0 )
    comp = 1/2 * comp;
end

end
