% Project irreducible tensors onto scalar spherical harmonics

% INPUT:  D = deformation matrix
%         E = rate of strain matrix

% OUTPUT:  proj_Sd1 = < G_ipp ri, Y_1m >
%          proj_Sd2 = < Sd2[Eik Dkj] ri rj, Y_2m>
%          proj_Sd3 = < Sd3[G_ijk] ri rj rk, Y_3m >
%          proj_Sd4 = < Sd4[Eij Dkp] ri rj rk rp, Y_4m >
%                where G_ijk = eps_ikq Ekj Dqk
%                and Sd2[..], Sd3[..], Sd4[..] are symmetric, traceless deviators


function [proj_Sd1, proj_Sd2, proj_Sd3, proj_Sd4] = project_irreducible(D,E)

% Initialize projections over invariants
proj_Sd1 = zeros(3,1);
proj_Sd2 = zeros(5,1);
proj_Sd3 = zeros(7,1);
proj_Sd4 = zeros(9,1);

% Get invariants
Sd4_ED = Sd4(E, D);
[Sd1_ED, Sd3_ED] = get_pseudo_tensors(E, D);
Sd2_ED = Sd2(E*D);

% Get values of theta and phi for integration
N_theta = 20;
N_phi = 20;
[nodes, weights] = get_Gaussian_quad_weights(N_theta);
theta = acos(nodes);
phi = 2*pi/(2*N_phi) * (1:2:(2*N_phi-1));
d_phi = 2*pi/N_phi;

% Loop through and integrate
for i = 1:length(theta)
    for j = 1:length(phi)
       
        % Calculate r_hat
        r_hat = [sin(theta(i)) * cos(phi(j));
                 sin(theta(i)) * sin(phi(j));
                 cos(theta(i))];
             
         % Get spherical harmonics
        Y_one_m = get_1st_ord_harmonics(theta(i), phi(j));
        Y_2m = get_2nd_ord_harmonics(theta(i), phi(j));        
        Y_3m = get_3rd_ord_harmonics(theta(i), phi(j));        
        Y_4m = get_4th_ord_harmonics(theta(i), phi(j));        


        % Get associated scalars for invariants
        scalar_Sd4 = 0;
        scalar_Sd3 = 0;
        scalar_Sd2 = r_hat' * (Sd2_ED * r_hat);
        scalar_Sd1 = r_hat' * Sd1_ED;
        for k = 1:3
            for m = 1:3
                for n = 1:3
                    scalar_Sd3 = scalar_Sd3 + Sd3_ED(k,m,n) * r_hat(k) * r_hat(m) * r_hat(n);
                    for p = 1:3
                        scalar_Sd4 = scalar_Sd4 + Sd4_ED(k,m,n,p) * r_hat(k) * r_hat(m) * r_hat(n) * r_hat(p);
                    end
                end
            end
        end
        
        % Project scalars onto harmonics
        proj_Sd1 = proj_Sd1 + weights(i) * d_phi * Y_one_m * scalar_Sd1;
        proj_Sd2 = proj_Sd2 + weights(i) * d_phi * Y_2m * scalar_Sd2;
        proj_Sd3 = proj_Sd3 + weights(i) * d_phi * Y_3m * scalar_Sd3;
        proj_Sd4 = proj_Sd4 + weights(i) * d_phi * Y_4m * scalar_Sd4;
                
    end
end

end