% Peturb surface by O(Ca) and project capillary pressure onto harmonics
% (2nd or 4th order)
% INPUT:  D = 2nd order deformation matrix
%         D4 = 4th order deformation matrix
%         flag_proj = flag for harmonics we project BCs.  flag_proj = 2 if we project onto 2nd order harmonics, flag_proj = 4 if
%                     we project onto fourth order harmonics

% OUTPUT:
% If projecting onto 2nd and 4th order harmonics,
% comp(:, 1) = components of kinematic BC
% comp(:, 2) = tangential component of [[ui]]
% comp(:, 3) = normal component of [[ui]]
% comp(:, 4) = tangential part of stress
% comp(:, 5) = tangential part of fvis
% comp(:, 6) = tangential part of fdil
% comp(:, 7) = normal part of stress
% comp(:, 8) = normal part of fvis
% comp(:, 9) = normal part of fdil

% For second order harmonics:  rows are (20, 21, 2-1, 22, 2-2) components 
% For fourth order harmonics:  rows are (40, 41, 4-1, 42, 4-2, 43, 4-3, 44, 4-4) components

% Note:  whenever we project onto vector harmonics, we multiply by a factor of
% 1/(n+1) so that we can compare projections.  The
% reason is that:
%          < Aqjmp... rq rj rm rp ..., Y_nm> = 1/(n+1) * < (Iiq - ri rq) * Aqjmp... rj rm rp ..., Psi_nm> =  .
%                                            = 1/(n+1) * < eps_ijk rj * Akmp... rm rp ..., Phi_nm> 

% The above will useful when we want to calculate the coefficient in front of one of the boundary
% conditions in flow, e.g.,
%               tang_vel = coef * (Iip - ri rp) * Sd4[Dpj Dkq] * rj rk rq
% In this example, the projection of the above with the fourth order vector harmonic Psi_4m gives
%               <tang_vel, Psi_4m> = coef * < (Iip - ri rp) * Sd4[Dpj Dkq] rj rk rq, Psi_4m> = coef * 5 * < Sd4[Dpj Dkq] rp rj rk rq, Y_4m>

% Thus, coef = 1/5 * <tang_vel, Psi_4m>/<Sd4[Dpj Dkq] rp rj rk rq, Y_4m>.
% If we looked at something like the normal velocity where we project onto
% Y_4m rather than a vector harmoic, we will no longer have the factor of
% 1/5 ( i.e., 1/(n+1) )

% OTHER OUTPUT:
% If projecting onto 2nd order harmonics
% coef(1) = coef of kinematic BC for this flow, i.e., kinematic_BC = coef(1) * Sd2[D*D] * r * r 
% coef(2) = coef of tangential component of [[ui]] for this flow, i.e., tang_comp = coef(2) * (I - rr) * Sd2[D*D] * r
% coef(3) = coef of normal component of [[ui]], i.e., norm_comp = coef(3) * r * Sd2[D*D] * r
% coef(4) = coef for tangential part of stress, i.e., tang_stress = coef(4) * (I - rr) * Sd2[D*D] * r
% coef(5) = coef for tangential part of fvis, i.e., tang_f_vis = coef(5) * (I-rr) * Sd2[D*D] * r
% coef(6) = coef for tangential part of fdil, i.e., tang_f_dil = coef(6) * (I-rr) * Sd2[D*D] * r
% coef(7) = coef for normal part of stress, i.e., norm_stress = coef(7) * r * Sd2[D*D] * r
% coef(8) = coef for normal part of fvis, i.e., norm_f_vis = coef(8) * r * Sd2[D*D] * r
% coef(9) = coef for normal part of fdil, i.e., norm_f_dil = coef(9) * r * Sd2[D*D] * r

% If projecting onto fourth order harmonics, we compute the coefficients for the
% tensor Sd4[DD] -- i.e., replace the Sd2[Dik Dkj] in the above expressions with Sd4[Dik Dmj] * rk rm


function [comp, coef] = get_coefs_cap_pressure_updated(D, D4, flag_proj)

% Add path
%addpath('Harmonics', 'Evaluate_BCs');

% The capillary pressure only modifies normal stress balance
m_modes = 2*flag_proj + 1;
stress_norm_proj = zeros(m_modes,1);

% For each value of theta and phi, calculate velocity field to O(Ca)
N_theta = 20;
N_phi = 20;
[nodes, weights] = get_Gaussian_quad_weights(N_theta);
theta = acos(nodes);
phi = 2*pi/(2*N_phi) * (1:2:(2*N_phi-1));
d_phi = 2*pi/N_phi;

for i = 1:length(theta)
    for j = 1:length(phi)
        
        
        % For value of theta and phi, calculate boundary conditions on
        % deformed surface to appropriate order of Ca.  We will perform
        % finite difference schemes to evaluate derivatives
        Ca = 5e-4;

        % Perform finite difference to get capillary pressure at O(Ca)
        [~, ~, ~, ~, ~, ~, dH_right] = update_geom(D, D4, 0.5*Ca, theta(i), phi(j));
        [~, ~, ~, ~, ~, ~, dH_left] = update_geom(D, D4, -0.5*Ca, theta(i), phi(j));
        n_sigma_n = -2 * (dH_right - dH_left)/Ca;
                   
        % Project boundary conditions onto spherical harmonics
        if( flag_proj == 2 )  % Projecting onto 2nd order harmonics
            Ylm = get_2nd_ord_harmonics(theta(i), phi(j));
        elseif( flag_proj == 4 )
            Ylm = get_4th_ord_harmonics(theta(i), phi(j));
        else
            error('flag_proj must be 2 or 4');
        end
        stress_norm_proj = stress_norm_proj + weights(i) * d_phi * n_sigma_n * Ylm;
        
    end
end

% Gather projection of all BCs into final result  
comp = zeros(m_modes, 9);
comp(:, 7) = -stress_norm_proj;


% Normalize projected BCs by < Eijk... xi xj xk..., Ynm>.  
% For second order harmonics:  For ith BC, the modes comp(:,i) = coef(i) * proj_Sd2
% For fourth order harmonics:  For ith BC, the modes comp(:,i) = coef(i) * proj_Sd4
% Where proj_Sd2 = <Sd2[Lik Dkj] ri rj, Y2m>
%       proj_Sd4 = <Sd4[Lij Dkp] ri rj rk rp, Y4m>
[~, proj_Sd2, ~, proj_Sd4] = project_irreducible(D,D);
if( flag_proj == 2)
    factor = proj_Sd2;
else
    factor = proj_Sd4;
end

num_BCs = length(comp(1,:));
coef = zeros(num_BCs, 1);
for i = 1:num_BCs
    coef(i) = (factor' * comp(:,i) )/(factor' * factor);
end

end
