% function:  coefs_A_D_ord_Ca
% get coefs A-D for flows up to O(Ca

% INPUT:  lambda = viscosity ratio
%         Bq_k = dilatational Boussinesq number
%         Bq_mu = shear Boussinesq number

% OUTPUT:  c_E      = [A,B,C,D] for flow driven by Eij
%          c_D      = [A,B,C,D] for flow driven by Dij
%          c_Sd2_ED = [A,B,C,D] for flow driven by Sd2(ED)
%          c_Sd2_DD = [A,B,C,D] for flow driven by Sd2(DD)
%          c_D4     = [P,Q,R,S] for flow driven by Dijkm
%          c_Sd4_ED = [P,Q,R,S] for flow driven by Sd4(ED)
%          c_Sd4_DD = [P,Q,R,S] for flow driven by Sd4(DD)
%          c_Sd3_ED = [A, B] for flow driven by Sd3[G_ijk]
%          c_Sd1_ED = [A, B] for flow driven by G_ipp
%                                   G_ijk = eps_ipq Epj Dqk

function [c_E, c_D, c_Sd2_ED, c_Sd2_DD, ...
          c_D4, c_Sd4_ED, c_Sd4_DD, ...
          c_Sd3_ED, c_Sd1_ED] = compute_coefs_ord_Ca(lambda, Bq_k, Bq_mu)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 2nd order harmonics %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% Get Sd2(ED) term %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First, get solution to O(1) velocity excited by E.  sol = coefs [A,B,C,D] at O(1)

% Set up matrices
M = [             0,   0,                               3,                             1,    -1;
                  0,   2,                              -5,                            -1,     0;
                 -3,  -3,                              -3,                            -1,     0;
                 -6, -16, -16*lambda - 18*Bq_k - 20*Bq_mu,  -2*lambda - 2*Bq_k - 4*Bq_mu,     0;
                 18,  24,              3*lambda + 18*Bq_k,            -2*lambda + 2*Bq_k,     0];
   
b = [ 0;
     -1;
     -1;
     -2;
     -2];
 
sol_E = M \ b;
c_E = sol_E(1:4);

% Now, get coefs to O(Ca) velocity disturbed by ED
% M * c(1) = 2/7 * ( F * c(0) + b )
% where M and F are matrices, b is a vector, and c(0) are the coefficients
% for the O(1) velocity field linear in E

F = [   0,    0,                                -3,                               1;
        6,   14,                                21,                               3;
      -12,  -18,                                 3,                              -1;
      -54, -128,    26*lambda - 28*Bq_mu - 42*Bq_k,   -6*lambda - 4*Bq_mu - 14*Bq_k;
       72,  144, -108*lambda + 120*Bq_mu - 48*Bq_k,  -12*lambda + 24*Bq_mu + 4*Bq_k];

b = [0, -3, 1, 6, 12]';

rhs = 2/7 * ( F * c_E + b );
coefs_ED = M \ rhs;
c_Sd2_ED = coefs_ED(1:4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% Get Sd2(DD) term %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First, get solution to O(1) velocity excited by D.  sol = coefs [A,B,C,D] at O(1)
% Set up matrices
M = [             0,   0,                               3,                             1,    -1;
                  0,   2,                              -5,                            -1,     0;
                 -3,  -3,                              -3,                            -1,     0;
                 -6, -16, -16*lambda - 18*Bq_k - 20*Bq_mu,  -2*lambda - 2*Bq_k - 4*Bq_mu,     0;
                 18,  24,              3*lambda + 18*Bq_k,            -2*lambda + 2*Bq_k,     0];
   
b = [ 0;
      0;
      0;
      0;
      4];
 
sol_D = M \ b;
c_D = sol_D(1:4);

% Now, get coefs to O(Ca) velocity disturbed by DD
% M * c(1) = 2/7 * ( F * c(0) + b )
% where M and F are matrices, b is a vector, and c(0) are the coefficients
% for the O(1) velocity field linear in D

F = [   0,    0,                                -3,                               1;
        6,   14,                                21,                               3;
      -12,  -18,                                 3,                              -1;
      -54, -128,    26*lambda - 28*Bq_mu - 42*Bq_k,   -6*lambda - 4*Bq_mu - 14*Bq_k;
       72,  144, -108*lambda + 120*Bq_mu - 48*Bq_k,  -12*lambda + 24*Bq_mu + 4*Bq_k];

b = [0, 0, 0, 0, -20]';

rhs = 2/7 * ( F * c_D + b );
coefs_DD = M \ rhs;
c_Sd2_DD = coefs_DD(1:4);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% 4th order harmonics %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% Get D_4 term %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First, get solution to O(Ca) velocity excited by D_4.  sol = coefs [A,B,C,D, u_n] at O(Ca)
% Set up matrices
G = [ 0,   0,                                 -20,                                4,     -1;
      2,   4,                                  28,                               -4,      0;
     -5,  -5,                                  20,                               -4,      0;
     -30, -48,  192*lambda + 400*Bq_k + 504*Bq_mu,  -24*lambda - 48*Bq_k - 72*Bq_mu,      0;
     54,  60,                90*lambda - 200*Bq_k,             -24*lambda + 24*Bq_k,      0];
   
b = [ 0;
      0;
      0;
      0;
      18];
 
coefs_D_4 = G \ b;
c_D4 = coefs_D_4(1:4);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% Get Sd4(ED) term %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Now, get coefs for O(Ca) velocity disturbed by Sd4(ED)
% G * c(1) = H * c(0) + b
% where G and H are matrices, b is a vector, and c(0) are the coefficients
% for the O(1) velocity field linear in E

H = [  0,     0,                                -19,  -3;
       6,     14,                                21,   3;
      -6,   -16,                                 19,   3;
     -54,  -156, 96*lambda + 252*Bq_mu + 182*Bq_k,  8*lambda + 24*Bq_mu + 14*Bq_k;
      78,   184,   58*lambda - 10*Bq_mu - 136*Bq_k,  8*lambda - 2*Bq_mu - 12*Bq_k];

b = [0, -3, -3, -8, -8]';  

rhs = ( H * c_E + b );
coefs_ED = G \ rhs;
c_Sd4_ED = coefs_ED(1:4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% Get Sd4(DD) term %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Now, get coefs to O(Ca) velocity disturbed by Sd4(DD)
% G * c(1) = H * c(0) + b
% where G and H are matrices, b is a vector, and c(0) are the coefficients
% for the O(1) velocity field linear in D

H = [  0,     0,                                -19,  -3;
       6,     14,                                21,   3;
      -6,   -16,                                 19,   3;
     -54,  -156, 96*lambda + 252*Bq_mu + 182*Bq_k,  8*lambda + 24*Bq_mu + 14*Bq_k;
      78,   184,   58*lambda - 10*Bq_mu - 136*Bq_k,  8*lambda - 2*Bq_mu - 12*Bq_k];

b = [0, 0, 0, 0, -10]';

rhs = ( H * c_D + b );
coefs_DD = G \ rhs;
c_Sd4_DD = coefs_DD(1:4);






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% 3rd order harmonics %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Get Sd3[G_ijk] term, where Gijk = eps_imp Ejm Dkp %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Now, get coefs to O(Ca) velocity disturbed by Sd3[G_ijk]
% W * c(1) = 1/2 * (J * c(0) + b)
% where W and J are matrices, b is a vector, and c(0) are the coefficients
% for the O(1) velocity field linear in E

W = [ 1,                    -1;
     -5,  -2*lambda - 10*Bq_mu];
 
J = [ 0,   8,                             15,                           1;
    -18, -68, 2*lambda + 140*Bq_mu - 54*Bq_k, 4*lambda + 8*Bq_mu - 6*Bq_k];

b = [-1, -4]';

rhs = 1/2 * ( J * c_E + b );
c_Sd3_ED = W \ rhs;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% 1st order harmonics %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Get G_ipp term, where Gijk = eps_imp Ejm Dkp %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Now, get coefs to O(Ca) velocity disturbed by G_ipp
% W * c(1) = 1/5 * (J * c(0) + b)
% where W and J are matrices, b is a vector, and c(0) are the coefficients
% for the O(1) velocity field linear in E

W = [ 1, -1;
     -3,  0];
 
J = [ 0,   8,                             15,                           1;
    -18, -48, -48*lambda - 60*Bq_mu - 54*Bq_k, -6*lambda - 12*Bq_mu - 6*Bq_k];

b = [-1, 6]';

rhs = 1/5 * ( J * c_E + b );
c_Sd1_ED = W \ rhs;
 

end