% function:  get_vel_coefs_order_Ca
% We get the linear system of equations for O(Ca)
% Linear system is of the form M*x = H*c + b

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%% For second order and fourth order harmonic contribution:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% x = coefs [A, B, C, D] for O(Ca) velocity field
% M = matrix such that M*x gives contribution to BCs for O(Ca) vel field
% right hand side = contribution to BC from O(1) vel field perturbed by the deformed surface
% H*c = contribution from O(1) disturbance velocity, c = [A, B, C, D] for O(1) disturbance field
% b = contribution from O(1) far-field velocity and capillary pressure

% We calculate the matrix H and vector b

% We use a slightly different convection for writing H and b so that we can
% later do symbolic manipulation

% row 1 = contribution to kinematic
% row_2 = contribution to P*[[u]]
% row_3 = contribution to n*[[u]]
% row_4 = contribution to P * [[sigma * n]]
% row_5 = contribution to P * f_vis
% row_6 = contribution to P * f_dil
% row_7 = contribution to n * [[sigma * n]]
% row_8 = contribution to n * f_vis
% row_9 = contribution to n * f_dil


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%% For first and third order harmonic contributions: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% x = coefs [A, B] for O(Ca) velocity field
% M = matrix such that M*x gives contribution to BCs for O(Ca) vel field
% right hand side = contribution to BC from O(1) vel field perturbed by the deformed surface
% H*c = contribution from O(1) disturbance velocity, c = [A, B, C, D] for O(1) disturbance field
% b = contribution from O(1) far-field velocity and capillary pressure

% We calculate the matrix H and vector b

% We use a slightly different convection for writing H and b so that we can
% later do symbolic manipulation

% row_1 = contribution to [[u]]
% row_2 = contribution to [[sigma * n]]
% row_3 = contribution to f_vis
% row_4 = contribution to f_dil

function [H_ED_second, b_ED_second, b_DD_second, ...
          H_ED_fourth, b_ED_fourth, b_DD_fourth, ...
          H_ED_third, b_ED_third, H_ED_first, b_ED_first] = get_linear_system_ord_Ca_updated_v2()

      
% Add path
addpath(genpath('Project_BCs_onto_modes'));


% Choose random matrices for D and E
D = rand(3,3);     D = Sd2(D);
E = rand(3,3);     E = Sd2(E);



% At O(Ca), we calculate coefficients for various flows.  For second and order harmonics:
% coef(1) = coef of kinematic BC, i.e., kinematic_BC = coef(1) * Sd2[L*D] * r * r 
% coef(2) = coef of tangential component of [[ui]], i.e., tang_comp = coef(2) * (I - rr) * Sd2[L*D] * r
% coef(3) = coef of normal component of [[ui]], i.e., norm_comp = coef(3) * r * Sd2[L*D] * r
% coef(4) = coef for tangential part of stress, i.e., tang_stress = coef(4) * (I - rr) * Sd2[L*D] * r
% coef(5) = coef for tangential part of fvis, i.e., tang_f_vis = coef(5) * (I-rr) * Sd2[L*D] * r
% coef(6) = coef for tangential part of fdil, i.e., tang_f_dil = coef(6) * (I-rr) * Sd2[L*D] * r
% coef(7) = coef for normal part of stress, i.e., norm_stress = coef(7) * r * Sd2[L*D] * r
% coef(8) = coef for normal part of fvis, i.e., norm_f_vis = coef(8) * r * Sd2[L*D] * r
% coef(9) = coef for normal part of fdil, i.e., norm_f_dil = coef(9) * r * Sd2[L*D] * r

% For fourth order harmonics, we compute the coefficients for the
% tensor Sd4[LD] -- i.e., replace the Sd2[Lik Dkj] in the above expressions with Sd4[Lik Dmj] * rk rm

% For first order harmonics
% coef(1) = coef of [[u]], i.e., [[u]] = coef(1) * eps(i,j,k) * rj * G_kmm 
% coef(2) = coef of jump in traction [[sigma_ij n_j]], i.e., traction = coef(2) * eps(i,j,k) * rj G_kmm
% coef(3) = coef of fvis, i.e., f_vis = coef(3) * eps(i,j,k) * rj * G_kmm
% coef(4) = coef for fdil, i.e., f_dil = coef(4) * eps(i,j,k) * rj * G_kmm
% In the above formulae, G_ijk = eps(i,m,p) Emj Dpk

% For third order harmonics, we compute the coefficients for the
% tensor Sd3[G_ijk] -- i.e., replace the G_kmm in the above expressions with Sd3[G_kmp] * rm * rp 








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% O(Ca) flow driven by Sd2(ED)  and Sd2[DD] %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get coefficients for second order tensor
H_ED_second = zeros(9, 4);
for j = 1:4
    coef_ord_one = zeros(4,1);
    coef_ord_one(j) = 1;
    flag_proj = 2;
    [~, coef] = get_coefs_disturbance_field_all_ord(E, D, zeros(3,3,3,3), coef_ord_one, flag_proj);
    H_ED_second(:,j) = coef;
end

% Get coef b for external flow
flag_proj = 2;
[~, b_ED_second] = get_coefs_far_field_all_ord(E, D, zeros(3,3,3,3), flag_proj);

% Get coef b for capillary pressure
flag_proj = 2;
[~, b_DD_second] = get_coefs_cap_pressure_updated(D, zeros(3,3,3,3), flag_proj);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% O(Ca) flow driven by Sd4(ED)  and Sd4[DD] %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get coefficients for second order tensor
H_ED_fourth = zeros(9, 4);
for j = 1:4
    coef_ord_one = zeros(4,1);
    coef_ord_one(j) = 1;
    flag_proj = 4;
    [~, coef] = get_coefs_disturbance_field_all_ord(E, D, zeros(3,3,3,3), coef_ord_one, flag_proj);
    H_ED_fourth(:,j) = coef;
end

% Get coef b for external flow
flag_proj = 4;
[~, b_ED_fourth] = get_coefs_far_field_all_ord(E, D, zeros(3,3,3,3), flag_proj);

% Get coef b for capillary pressure
flag_proj = 4;
[~, b_DD_fourth] = get_coefs_cap_pressure_updated(D, zeros(3,3,3,3), flag_proj);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% O(Ca) flow driven by eps(i,j,k) Ejm Dkm   %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get coefficients for second order tensor
H_ED_first = zeros(4, 4);
for j = 1:4
    coef_ord_one = zeros(4,1);
    coef_ord_one(j) = 1;
    flag_proj = 1;
    [~, coef] = get_coefs_disturbance_field_all_ord(E, D, zeros(3,3,3,3), coef_ord_one, flag_proj);
    H_ED_first(:,j) = coef;
end

% Get coef b for external flow
flag_proj = 1;
[~, b_ED_first] = get_coefs_far_field_all_ord(E, D, zeros(3,3,3,3), flag_proj);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% O(Ca) flow driven by Sd3[eps(i,m,p) Ejm Dkp]   %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get coefficients for second order tensor
H_ED_third = zeros(4, 4);
for j = 1:4
    coef_ord_one = zeros(4,1);
    coef_ord_one(j) = 1;
    flag_proj = 3;
    [~, coef] = get_coefs_disturbance_field_all_ord(E, D, zeros(3,3,3,3), coef_ord_one, flag_proj);
    H_ED_third(:,j) = coef;
end

% Get coef b for external flow
flag_proj = 3;
[~, b_ED_third] = get_coefs_far_field_all_ord(E, D, zeros(3,3,3,3), flag_proj);


end