% Evaulate far-field velocity fields and pressure fields on deformed surface

% INPUT:  E = rate of strain tensor
%         x = coordinate on surface
%         n = normal vector on surface

% Note:  the far-field velocity satisfy:
%      u_i^out = Eij * xj
%      p^out = 0

% OUTPUT:  P_u = P * (uout - uin) = jump in tangential disturbance velocity field across deformed surface
%          n_u = n * (uout - uin) = jump in normal disturbance velocity field across deformed surface 
%          kin_BC = uin*n = normal velocity on deformed surface
%          P_sigma_n = [P*sigma_out*n - P*sigma_in*n] =
%                      jump in tangential disturbance traction across deformed surface
%          n_sigma_n = n*[sigma_out - sigma_in]*n = jump in normal disturbance
%                                                   traction across deformed surface
%          u = (uout - uin) = jump in disturbance velocity across deformed surface
%          traction = jump in traction across the deformed surface


% Note:  du_out(i,j) = du_i/dx_j
%        ddu_out(i,j,k) = du_i / dx_j dx_k


function [P_u, n_u, kin_BC, P_sigma_n, n_sigma_n, u, traction] = eval_BCs_far_field(E, x, n)

% Get outer velocity field and pressures
u = E*x;
du_out = E;
p = 0;

% Get velocity jump at surface and kinematic BC
P = eye(3) - n * n';
P_u = P * u;
n_u = n' * u;
kin_BC = 0;

% Evaluate jump in stress tensor
sigma_out = du_out + du_out' - p * eye(3);
traction = sigma_out * n;
P_sigma_n = P * traction;
n_sigma_n = n' * traction;

end

