% Get third order harmonics
% INPUT:  theta = polar coordinate (0 to pi)
%         phi = azimuthal coordinate (0 to 2*pi)
% OUTPUT:  Ylm = 7 x 1 vector of scalar harmonics (20, 21, 2-1, 22, 2-2)
%          Psi_lm = 7 x 3 vector of vector harmonics grad(Ylm)
%          Phi_lm = 7 x 3 vector of vector harmonics r x grad(Ylm)
function [Ylm, Psi_lm, Phi_lm] = get_3rd_ord_harmonics(theta, phi)

% First, define unit vectors
r_hat = [sin(theta) * cos(phi), sin(theta) * sin(phi), cos(theta)];
theta_hat = [cos(theta) * cos(phi), cos(theta) * sin(phi), -sin(theta)];
phi_hat = [-sin(phi), cos(phi), 0];
        
% Zero quantities
Ylm = zeros(7,1);
Psi_lm = zeros(7, 3);
Phi_lm = zeros(7, 3);

% Define harmonic functions
% Y30
Ylm(1) = 1/4 * sqrt(7/pi) * ( 5*cos(theta)^3 - 3*cos(theta) );
Psi_lm(1, :) = -1/4 * sqrt(7/pi) * ( 15 * cos(theta)^2 * sin(theta) - 3 * sin(theta) ) * theta_hat;

% Y3-3
Ylm(2) = 1/8 * sqrt(70/pi) * sin(3*phi) * sin(theta)^3;
Psi_lm(2, :) = 1/8 * sqrt(70/pi) * ( 3 * sin(theta)^2 * cos(theta) * sin(3*phi) * theta_hat + 3*cos(3*phi) * sin(theta)^2 * phi_hat );        

% Y3-2
Ylm(3) = 1/4 * sqrt(105/pi) * sin(2*phi) * sin(theta)^2 * cos(theta);
Psi_lm(3, :) = 1/4 * sqrt(105/pi) * ( sin(theta) * (3*cos(theta)^2 - 1) * sin(2*phi) * theta_hat + 2*cos(2*phi) * sin(theta) * cos(theta) * phi_hat );        

% Y3-1
Ylm(4) = 1/8 * sqrt(42/pi) * sin(phi) * sin(theta) * (5 * cos(theta)^2 - 1);
Psi_lm(4, :) = 1/8 * sqrt(42/pi) * ( cos(theta) * ( 15*cos(theta)^2 - 11 ) * sin(phi) * theta_hat + cos(phi) * ( 5* cos(theta)^2 - 1) * phi_hat );        


% Y33
Ylm(5) = -1/8 * sqrt(70/pi) * cos(3*phi) * sin(theta)^3;
Psi_lm(5, :) = -1/8 * sqrt(70/pi) * ( 3 * sin(theta)^2 * cos(theta) * cos(3*phi) * theta_hat - 3*sin(3*phi) * sin(theta)^2 * phi_hat );        

% Y32
Ylm(6) = 1/4 * sqrt(105/pi) * cos(2*phi) * sin(theta)^2 * cos(theta);
Psi_lm(6, :) = 1/4 * sqrt(105/pi) * ( sin(theta) * (3*cos(theta)^2 - 1) * cos(2*phi) * theta_hat - 2*sin(2*phi) * sin(theta) * cos(theta) * phi_hat );        

% Y31
Ylm(7) = -1/8 * sqrt(42/pi) * cos(phi) * sin(theta) * (5 * cos(theta)^2 - 1);
Psi_lm(7, :) = -1/8 * sqrt(42/pi) * ( cos(theta) * ( 15*cos(theta)^2 - 11 ) * cos(phi) * theta_hat - sin(phi) * ( 5* cos(theta)^2 - 1) * phi_hat );        


% Get Phi_lm
for i = 1:7
    Phi_lm(i, :) = cross(r_hat, Psi_lm(i, :));
end

end