% Get second order harmonics
% INPUT:  theta = polar coordinate (0 to pi)
%         phi = azimuthal coordinate (0 to 2*pi)
% OUTPUT:  Ylm = 9 x 1 vector of scalar harmonics (40, 41, 4-1, 42, 4-2, 43, 4-3, 44, 4-4)
%          Psi_lm = 9 x 3 vector of vector harmonics grad(Ylm)
%          Phi_lm = 9 x 3 vector of vector harmonics r x grad(Ylm)
function [Ylm, Psi_lm, Phi_lm] = get_4th_ord_harmonics(theta, phi)

% First, define unit vectors
r_hat = [sin(theta) * cos(phi), sin(theta) * sin(phi), cos(theta)];
theta_hat = [cos(theta) * cos(phi), cos(theta) * sin(phi), -sin(theta)];
phi_hat = [-sin(phi), cos(phi), 0];
        
% Zero quantities
Ylm = zeros(9,1);
Psi_lm = zeros(9, 3);
Phi_lm = zeros(9, 3);

% Define harmonic functions
% Y40
Ylm(1) = 3/16 * sqrt(1/pi) * (35 * cos(theta)^4 - 30*cos(theta)^2 + 3);
Psi_lm(1, :) = 3/16 * sqrt(1/pi) * sin(2*theta) * (-35*2*cos(theta)^2 + 30) * theta_hat;

% Y41
Ylm(2) = -3/8 * sqrt(10/pi) * cos(phi) * sin(theta) * cos(theta) * (7*cos(theta)^2 - 3);
Psi_lm(2, :) = -3/8*sqrt(10/pi) * ( -sin(phi)*cos(theta)*(7*cos(theta)^2 - 3)*phi_hat + cos(phi) * (cos(2*theta) * (7*cos(theta)^2 - 3) - 7/2 * sin(2*theta)^2) * theta_hat );        

% Y4-1
Ylm(3) = 3/8 * sqrt(10/pi) * sin(phi) * sin(theta) * cos(theta) * (7*cos(theta)^2 - 3);
Psi_lm(3, :) = 3/8*sqrt(10/pi) * ( cos(phi)*cos(theta)*(7*cos(theta)^2 - 3)*phi_hat + sin(phi) * (cos(2*theta) * (7*cos(theta)^2 - 3) - 7/2 * sin(2*theta)^2) * theta_hat );

% Y42
Ylm(4) = 3/8*sqrt(5/pi) * cos(2*phi) * sin(theta)^2 * (7*cos(theta)^2 - 1);        
Psi_lm(4, :) = 3/8*sqrt(5/pi) * ( -2*sin(2*phi) * sin(theta) * (7*cos(theta)^2 - 1) * phi_hat + cos(2*phi) * sin(2*theta) * (7*cos(2*theta) - 1) * theta_hat );

% Y4-2
Ylm(5) = 3/8*sqrt(5/pi) * sin(2*phi) * sin(theta)^2 * (7*cos(theta)^2 - 1);        
Psi_lm(5, :) = 3/8*sqrt(5/pi) * ( 2*cos(2*phi) * sin(theta) * (7*cos(theta)^2 - 1) * phi_hat + sin(2*phi) * sin(2*theta) * (7*cos(2*theta) - 1) * theta_hat );

% Y43
Ylm(6) = -3/8*sqrt(70/pi) * cos(3*phi) * sin(theta)^3 * cos(theta);
Psi_lm(6, :) = -3/8*sqrt(70/pi) * (-3*sin(3*phi) * sin(theta)^2 * cos(theta) * phi_hat + cos(3*phi) * sin(theta)^2 * (3*cos(theta)^2 - sin(theta)^2) * theta_hat );

% Y4-3
Ylm(7) = 3/8*sqrt(70/pi) * sin(3*phi) * sin(theta)^3 * cos(theta);
Psi_lm(7, :) = 3/8*sqrt(70/pi) * (3*cos(3*phi) * sin(theta)^2 * cos(theta) * phi_hat + sin(3*phi) * sin(theta)^2 * (3*cos(theta)^2 - sin(theta)^2) * theta_hat );

% Y44
Ylm(8) = 3/16 * sqrt(35/pi) * cos(4*phi) * sin(theta)^4;
Psi_lm(8, :) = 3/16 * sqrt(35/pi) * ( -4*sin(4*phi) * sin(theta)^3 * phi_hat + cos(4*phi) * 2 * sin(2*theta) * sin(theta)^2 * theta_hat );

% Y4-4
Ylm(9) = 3/16 * sqrt(35/pi) * sin(4*phi) * sin(theta)^4;
Psi_lm(9, :) = 3/16 * sqrt(35/pi) * ( 4*cos(4*phi) * sin(theta)^3 * phi_hat + sin(4*phi) * 2 * sin(2*theta) * sin(theta)^2 * theta_hat );

% Get Phi_lm
for i = 1:9
    Phi_lm(i, :) = cross(r_hat, Psi_lm(i, :));
end

end