% Perturb boundary by O(Ca) and project boundary conditions driven by
% far-field flow onto spherical harmonics (1st-4th order)
% INPUT:  E = tensor for external velocity field (2nd order tensor)
%         D = 2nd order deformation matrix
%         D4 = 4th order deformation matrix
%         flag_proj = flag for harmonics we project BCs.  flag_proj = 2 if we project onto 2nd order harmonics, flag_proj = 4 if
%                     we project onto fourth order harmonics, etc.


% OUTPUT:
% If projecting onto 2nd and 4th order harmonics,
% comp(:, 1) = components of kinematic BC
% comp(:, 2) = tangential component of [[ui]]
% comp(:, 3) = normal component of [[ui]]
% comp(:, 4) = tangential part of stress
% comp(:, 5) = tangential part of fvis
% comp(:, 6) = tangential part of fdil
% comp(:, 7) = normal part of stress
% comp(:, 8) = normal part of fvis
% comp(:, 9) = normal part of fdil

% If projecting onto 1st and 3rd order harmonics,
% comp(:, 1) = components of jump in velocity [[u]]
% comp(:, 2) = components of jump in stress
% comp(:, 3) = components of fvis
% comp(:, 4) = components of fdil


% For first order harmonics:  rows are (10, 11, 1-1) components 
% For second order harmonics:  rows are (20, 21, 2-1, 22, 2-2) components 
% For third order harmonics:  rows are (30, 3-3, 3-2, 3-1, 33, 32, 31) components
% For fourth order harmonics:  rows are (40, 41, 4-1, 42, 4-2, 43, 4-3, 44, 4-4) components

% Note:  whenever we project onto vector harmonics, we multiply by a factor of
% 1/(n+1) so that we can compare projections.  The
% reason is that:
%          < Aqjmp... rq rj rm rp ..., Y_nm> = 1/(n+1) * < (Iiq - ri rq) * Aqjmp... rj rm rp ..., Psi_nm> =  .
%                                            = 1/(n+1) * < eps_ijk rj * Akmp... rm rp ..., Phi_nm> 

% The above will useful when we want to calculate the coefficient in front of one of the boundary
% conditions in flow, e.g.,
%               tang_vel = coef * (Iip - ri rp) * Sd4[Epj Dkq] * rj rk rq
% In this example, the projection of the above with the fourth order vector harmonic Psi_4m gives
%               <tang_vel, Psi_4m> = coef * < (Iip - ri rp) * Sd4[Epj Dkq] rj rk rq, Psi_4m> = coef * 5 * < Sd4[Epj Dkq] rp rj rk rq, Y_4m>

% Thus, coef = 1/5 * <tang_vel, Psi_4m>/<Sd4[Epj Dkq] rp rj rk rq, Y_4m>.
% If we looked at something like the normal velocity where we project onto
% Y_4m rather than a vector harmoic, we will no longer have the factor of
% 1/5 ( i.e., 1/(n+1) )

% OTHER OUTPUT:
% If projecting onto 2nd order harmonics
% coef(1) = coef of kinematic BC for this flow, i.e., kinematic_BC = coef(1) * Sd2[E*D] * r * r 
% coef(2) = coef of tangential component of [[ui]] for this flow, i.e., tang_comp = coef(2) * (I - rr) * Sd2[E*D] * r
% coef(3) = coef of normal component of [[ui]], i.e., norm_comp = coef(3) * r * Sd2[E*D] * r
% coef(4) = coef for tangential part of stress, i.e., tang_stress = coef(4) * (I - rr) * Sd2[E*D] * r
% coef(5) = coef for tangential part of fvis, i.e., tang_f_vis = coef(5) * (I-rr) * Sd2[E*D] * r
% coef(6) = coef for tangential part of fdil, i.e., tang_f_dil = coef(6) * (I-rr) * Sd2[E*D] * r
% coef(7) = coef for normal part of stress, i.e., norm_stress = coef(7) * r * Sd2[E*D] * r
% coef(8) = coef for normal part of fvis, i.e., norm_f_vis = coef(8) * r * Sd2[E*D] * r
% coef(9) = coef for normal part of fdil, i.e., norm_f_dil = coef(9) * r * Sd2[E*D] * r

% If projecting onto fourth order harmonics, we compute the coefficients for the
% tensor Sd4[ED] -- i.e., replace the Sd2[Eik Dkj] in the above expressions with Sd4[Eik Dmj] * rk rm

% If projecting onto 1st order harmonics
% coef(1) = coef of [[u]] for this flow, i.e., u = coef(1) * eps(i,j,k) * rj * G_kmm 
% coef(2) = coef of jump in traction [[sigma_ij n_j]] for this flow, i.e., traction = coef(2) * eps(i,j,k) * rj G_kmm
% coef(3) = coef of fvis for this flow, i.e., f_vis = coef(3) * eps(i,j,k) * rj * G_kmm
% coef(4) = coef for fdil for this flow, i.e., f_dil = coef(4) * eps(i,j,k) * rj * G_kmm
% In the above formulae, G_ijk = eps(i,m,p) Emj Dpk

% If projecting onto third order harmonics, we compute the coefficients for the
% tensor Sd3[G_ijk] -- i.e., replace the G_kmm in the above expressions with Sd3[G_kmp] * rm * rp 

function [comp, coef] = get_coefs_far_field_all_ord(E, D, D4, flag_proj)

% Add path
% addpath('Harmonics', 'Evaluate_BCs');

% Initialize quantities
m_modes = 2*flag_proj + 1;
u_tang_proj = zeros(m_modes,1);             % For second and fourth order harmonics
u_n_proj = zeros(m_modes,1);
kin_proj = zeros(m_modes,1);
stress_tang_proj = zeros(m_modes,1);
stress_norm_proj = zeros(m_modes,1);

u_proj = zeros(m_modes, 1);                % For first and third order harmonics
stress_proj = zeros(m_modes, 1);


% For each value of theta and phi, calculate velocity field
N_theta = 20;
N_phi = 20;
[nodes, weights] = get_Gaussian_quad_weights(N_theta);
theta = acos(nodes);
phi = 2*pi/(2*N_phi) * (1:2:(2*N_phi-1));
d_phi = 2*pi/N_phi;

for i = 1:length(theta)
    for j = 1:length(phi)
        
        
        % For value of theta and phi, calculate boundary conditions on
        % deformed surface to appropriate order of Ca.  We will perform
        % finite difference schemes to evaluate derivatives
        Ca = 5e-4;

        % Right evaluation
        [n, x] = update_geom(D, D4, 0.5*Ca, theta(i), phi(j));
        [P_u_right, n_u_right, kin_right, P_sigma_n_right, n_sigma_n_right, u_right, traction_right] = eval_BCs_far_field(E, x, n);
            
        % Left evaluation
        [n, x] = update_geom(D, D4, -0.5*Ca, theta(i), phi(j));
        [P_u_left, n_u_left, kin_left, P_sigma_n_left, n_sigma_n_left, u_left, traction_left] = eval_BCs_far_field(E, x, n);

            
        % Finite difference for 2nd and 4th order harmonics
        if( flag_proj == 2 || flag_proj == 4 )
            
            % Finite differences to get BCs at O(Ca)
            P_u = (P_u_right - P_u_left)/Ca;
            n_u = (n_u_right - n_u_left)/Ca;
            kin = (kin_right - kin_left)/Ca;
            P_sigma_n = (P_sigma_n_right - P_sigma_n_left)/Ca;
            n_sigma_n = (n_sigma_n_right - n_sigma_n_left)/Ca;
        
            % Project onto spherical harmonics
            if( flag_proj == 2 )
                [Ylm, Psi_lm] = get_2nd_ord_harmonics(theta(i), phi(j));
            else
                [Ylm, Psi_lm] = get_4th_ord_harmonics(theta(i), phi(j));
            end
            
            u_tang_proj = u_tang_proj + weights(i) * d_phi * (Psi_lm * P_u);
            u_n_proj = u_n_proj + weights(i) * d_phi * n_u * Ylm;
            kin_proj = kin_proj + weights(i) * d_phi * kin * Ylm;
            stress_tang_proj = stress_tang_proj + weights(i) * d_phi * (Psi_lm * P_sigma_n);
            stress_norm_proj = stress_norm_proj + weights(i) * d_phi * n_sigma_n * Ylm;       
            
            
        else      % If projecting onto first and third harmonics
                        
            % Finite differences
            u = (u_right - u_left)/Ca;
            traction = (traction_right - traction_left)/Ca;
            
            % Project boundary conditions onto spherical harmonics
            if( flag_proj == 1 )   % 1st order harmonics
                [~, ~, Phi_lm] = get_1st_ord_harmonics(theta(i), phi(j));
            else                   % 3rd order harmonics
                [~, ~, Phi_lm] = get_3rd_ord_harmonics(theta(i), phi(j));
            end
            u_proj = u_proj + weights(i) * d_phi * (Phi_lm * u);
            stress_proj = stress_proj + weights(i) * d_phi * (Phi_lm * traction);

        end
    end
end


% Get projections onto harmonic components.  Note:  whenever we projected
% using vector harmonics, we add a normalization factor of 1/(n+1)
if( flag_proj == 2 || flag_proj == 4 )     % Second and fourth harmonics
    comp = zeros(m_modes, 9);
    comp(:, 1) = -kin_proj;
    comp(:, 2) = -u_tang_proj/(flag_proj + 1);
    comp(:, 3) = -u_n_proj;
    comp(:, 4) = -stress_tang_proj/(flag_proj + 1);
    comp(:, 7) = -stress_norm_proj;
else                 % First and third harmonics
    comp = zeros(m_modes, 4);
    comp(:, 1) = -u_proj;
    comp(:, 2) = -stress_proj;
    comp = comp/(flag_proj + 1);
end




% Normalize projected BCs by < Eijk... xi xj xk..., Ynm>.  
% For first order harmonics:  For ith BC, the modes comp(:,i) = coef(i) * proj_Sd1
% For second order harmonics:  For ith BC, the modes comp(:,i) = coef(i) * proj_Sd2
% etc...
% Where proj_Sd1 = <G_ipp ri, Y1m>
%       proj_Sd2 = <Sd2[Lik Dkj] ri rj, Y2m>
%       proj_Sd3 = <Sd3[Gijk] ri rj rk, Y3m>
%       proj_Sd4 = <Sd4[Lij Dkp] ri rj rk rp, Y4m>
%       Gijk = eps(i,q,p) Eqj Dpk

[proj_Sd1, proj_Sd2, proj_Sd3, proj_Sd4] = project_irreducible(D,E);
if( flag_proj == 1)
    factor = proj_Sd1;
elseif( flag_proj == 2)
    factor = proj_Sd2;
elseif( flag_proj == 3)
    factor = proj_Sd3;
else
    factor = proj_Sd4;
end

num_BCs = length(comp(1,:));
coef = zeros(num_BCs, 1);
for i = 1:num_BCs
    coef(i) = (factor' * comp(:,i) )/(factor' * factor);
end

end