% function:  update_geom
% This function gives all the geometric quantities associated with the
% droplet up to O(Ca^2)

% INPUT:  D = 2nd order deformation tensor (3x3)
%         D4 = 4th order deformation tensor (3x3x3x2)
%         Ca = capillary number,
%         theta = polar angle (from 0 to pi)
%         phi = azimuthal angle (from 0 to 2*pi)
% OUTPUT:  n = normal vector
%          x = vector on surface of droplet
%          r = radius of droplet
%          Dn = curvature tensor = dn_i/dn_j
%          H = mean curvature
%          P = projection matrix (i.e., I - nn)
%          grad_g = grad( r - r(theta, phi) ).  Used in kinematic BC

function [n, x, r, Dn, H, P, dH, grad_g] = update_geom(D, D4, Ca, theta, phi)

% Get radius of droplet
[r, x] = get_radius(Ca, theta, phi, D, D4);

% Get normal
[n, grad_g] = get_normal(x, Ca, D, D4);
P = eye(3) - n * n';   % Projection matrix

% Use central difference to get curvature tensor and mean curvature.  Need
% it to be accurate up to O(eps^3)
Dn = zeros(3,3);
eps = 5e-5;
for i = 1:3
    dx = zeros(3,1);
    dx(i) = eps;
%     n_rr = get_normal(x + 2*dx, Ca, D, D4);
%     n_r = get_normal(x + dx, Ca, D, D4);
%     n_ll = get_normal(x - 2*dx, Ca, D, D4);
%     n_l = get_normal(x - dx, Ca, D, D4);
%     Dn(:,i) = ( -n_rr + 8*n_r - 8*n_l + n_ll )/(12*eps);


    n_rrr = get_normal(x + 3*dx, Ca, D, D4);
    n_rr = get_normal(x + 2*dx, Ca, D, D4);
    n_r = get_normal(x + dx, Ca, D, D4);
    n_lll = get_normal(x - 3*dx, Ca, D, D4);
    n_ll = get_normal(x - 2*dx, Ca, D, D4);
    n_l = get_normal(x - dx, Ca, D, D4);
    Dn(:,i) = ( -n_lll + 9*n_ll - 45*n_l + 45*n_r - 9*n_rr + n_rrr )/(60*eps);

    
    
%    n_right = get_normal(x + 0.5*dx, Ca, D, D4);
%    n_left = get_normal(x - 0.5*dx, Ca, D, D4);
%    Dn(:,i) = (n_right - n_left)/eps;
end
Dn = P * Dn * P;
Dn = 0.5 * (Dn + Dn');   % Force it to be symmetric
%Dn = Dn * P;
H = 0.5 * trace(Dn);
dH = (H - 1)/Ca;

end





function [r, x] = get_radius(Ca, theta, phi, D, D4)

% Get unit normal on sphere
r_hat = [ sin(theta) * cos(phi);
          sin(theta) * sin(phi);
          cos(theta)];

% Get isotropic portion of radius to O(Ca^3)
DD = D*D;
DDD = DD * D;
r = 1 - 2/15 * Ca^2 * trace(DD) - 8/(3*3*5*7) * Ca^3 * trace(DDD);

% Get non-isotropic portion using second and fourth order tensors
for i = 1:3
    for j = 1:3
        r = r + Ca * r_hat(i) * r_hat(j) * D(i,j);
        for m = 1:3
            for k = 1:3
                r = r + Ca^2 * r_hat(i) * r_hat(j) * r_hat(m) * r_hat(k) * D4(i,j,m,k);
            end
        end
    end
end

% Get vector on surface of droplet
x = r * r_hat;

end



function [n, grad_g] = get_normal(x, Ca, D, D4)

% Get normal vector
r = norm(x);
r_hat = x/r;
n = r_hat;
P = eye(3) - n * n';   % Projection on unit sphere


% Get projection of D and D4 on sphere
PD = P * D;
PD4 = zeros(size(D4));
for i = 1:3
    for j = 1:3
        for m = 1:3
            for k = 1:3
                for q = 1:3
                    PD4(i,j,m,k) = PD4(i,j,m,k) + P(i,q) * D4(q,j,m,k);
                end
            end
        end
    end
end

for i = 1:3
    for j = 1:3
        n(i) = n(i) - 2 * Ca * PD(i,j) * r_hat(j)/r;
        for m = 1:3
            for k = 1:3
                n(i) = n(i) - 4 * Ca^2 * PD4(i,j,m,k) * r_hat(j) * r_hat(m) * r_hat(k)/r;
            end
        end
    end
end
grad_g = n;
n = n/norm(n);
    
end
    

